<?php
/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/

defined('ABSPATH') or die('You don\'t have access, silly');

if ( ! class_exists( 'GFForms' ) ) {
    die();
}

if ( ! class_exists('bizz_GravityFormsCwField')) {
    class bizz_GravityFormsCwField extends GF_Field
    {
        public $type;

        public function add_button( $field_groups )
        {
            $field_groups = $this->maybe_add_cw_field_group( $field_groups );

            return parent::add_button( $field_groups );
        }

        public function maybe_add_cw_field_group( $field_groups )
        {
            foreach ( $field_groups as $field_group ) {
                if ( $field_group['name'] == 'connectwise_fields' ) {
                    return $field_groups;
                }
            }

            $field_groups[] = array(
                'name'   => 'connectwise_fields',
                'label'  => __( 'ConnectWise Fields', 'gravitywise' ),
                'fields' => array()
            );

            return $field_groups;
        }

        public function is_conditional_logic_supported()
        {
            return true;
        }

        public function get_form_editor_button()
        {
            return array(
                'group' => 'connectwise_fields',
                'text'  => $this->get_form_editor_field_title()
            );
        }

        public function get_form_editor_field_settings()
        {
            return array(
                'conditional_logic_field_setting',
                'error_message_setting',
                'label_setting',
                'label_placement_setting',
                'admin_label_setting',
                'size_setting',
                'connectwise_components_standard_settings',
                'connectwise_components_advanced_settings',
                'rules_setting',
                'visibility_setting',
                'duplicate_setting',
                'description_setting',
            );
        }

        public function get_form_editor_inline_script_on_page_render()
        {
            // set the default field label for the simple type field
            $script = sprintf( "function SetDefaultValues_%s(field) {field.label = '%s';}", $this->type, $this->get_form_editor_field_title() ) . PHP_EOL;

            // initialize the fields custom settings
            $script .= "jQuery(document).bind('gform_load_field_settings', function (event, field, form) {" .
                       "var inputClass = field.inputClass == undefined ? '' : field.inputClass;" .
                       "jQuery('#input_class_setting').val(inputClass);" .
                       "});" . PHP_EOL;

            // saving the simple setting
            $script .= "function SetInputClassSetting(value) {SetFieldProperty('inputClass', value);}" . PHP_EOL;

            return $script;
        }

        public function get_value_merge_tag( $value, $input_id, $entry, $form, $modifier, $raw_value, $url_encode, $esc_html, $format, $nl2br )
        {
            $form_id = $form['id'];

            $getCwComponentData = gf_apply_filters( array('gravitywise_data_retrieval', $form_id), $this, $modifier);
            $text = $raw_value;

            if (!is_array($getCwComponentData) && !is_object($getCwComponentData)) {
                $text = str_replace( $value, $getCwComponentData, $raw_value );
            }

            return $text;
        }
    }
}