<?php
/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/

defined('ABSPATH') or die('You don\'t have access, silly');

GFForms::include_feed_addon_framework();

require_once ABSPATH . '/wp-content/plugins/gravitywise/includes/common/connectwise-client.php';

if ( !class_exists( 'bizz_GravityWiseFeedAddOn' ) ) {
    class bizz_GravityWiseFeedAddOn extends GFFeedAddOn
    {
        protected $_version = GRAVITYWISE_FEED_VERSION;
        protected $_min_version = '1.8.17';
        protected $_full_path = __FILE__;
        protected $_title = 'GravityWise Add-On';
        protected $_short_title = 'GravityWise Add-On';
        protected $_slug = 'gravitywise';
        protected $_path = 'gf-addon/gravitywise-feed-addon.php';

        /** @var  bizz_ConnectWiseClient API class instance*/
        protected $api;

        /** @var  object data retrieved from the cw_api*/
        protected $component_data;

        /** @var array  */
        protected $_capabilities = array('gravityformsinternal_connectwise', 'gravityformsinternal_connectwise_uninstall');

        /** @var string  */
        protected $_capabilities_settings_page = 'gravityformsinternal_connectwise';

        /** @var string  */
        protected $_capabilities_form_settings = 'gravityformsinternal_connectwise';

        /** @var string  */
        protected $_capabilities_uninstall = 'gravityformsinternal_connectwise_uninstall';

        /** @var bool  */
        protected $_enable_rg_autoupgrade = false;

        private static $_instance = null;

        /**
         * Get an instance of this class
         * 
         * @return bizz_GravityWiseFeedAddOn
         */
        public static function get_instance()
        {
            if (self::$_instance == null) {
                self::$_instance = new bizz_GravityWiseFeedAddOn();
            }

            return self::$_instance;
        }

        public function pre_init()
        {
            parent::pre_init();
            // add_filter( 'gform_addon_feed_settings_fields', array( $this, 'add_custom_field'), 10, 2 );
            add_filter('gform_addon_navigation', array($this, 'create_feed_retry_menu'));
            // add_filter('gform_addon_navigation', array($this, 'create_duplicate_check_menu'));
        }

        public function feed_settings_title()
        {
            return esc_html__('ConnectWise Feed Builder', 'gravitywise');
        }

        /**
         * Initialize ConnectWise feed add-on
         */
        public function init()
        {
            require_once ABSPATH . '/wp-content/plugins/gravitywise/includes/common/connectwise-client.php';

            parent::init();

            $settings = $this->get_plugin_settings();
            $this->api = new bizz_ConnectWiseClient();

            // initialize the component data variable as an array
            $this->component_data = array();
        }

        public function add_retry_button($meta_boxes, $entry, $form)
        {
            if ($this->get_active_feeds($form['id'])) {
                $meta_boxes[$this->_slug] = array(
                    'title'    => $this->get_short_title(),
                    'callback' => array($this, 'add_details_retry_meta_box'),
                    'context'  => 'side',
                );
            }
            return $meta_boxes;
        }

        public function create_feed_retry_menu($menus)
        {
            $menus[] = array('name' => 'feed_retrials', 'label' => __('GravityWise Feed History'), 'callback' =>  array($this, 'feed_retrial_callback'));
            return $menus;
        }

//        public function create_duplicate_check_menu($menus)
//        {
//            $menus[] = array('name' => 'duplicate_check', 'label' => __('Duplicate Checker'), 'callback' =>  array($this, 'duplicate_callback'));
//            return $menus;
//        }

//        public function duplicate_callback()
//        {
//            include ABSPATH . '/wp-content/plugins/gravitywise/public/js/vue/tests/duplicate-check.php';
//        }

		public function process_feed( $feed, $entry, $form ) {
			require_once ABSPATH . '/wp-content/plugins/gravitywise/includes/feeds/feed-dispatcher.php';
			$d = new bizz_FeedDispatcher(new bizz_ConnectWiseClient());
			$d->generate_new_entries();
			$d->process_entries();
		}

	    /**
         * Adding functionality for entry retry button
         */
        public function add_details_retry_meta_box($args)
        {
            echo 'Details';
        }

        /**
         * Fields that will be used for the feed settings page
         */
        public function feed_settings_fields()
        {


            $feed = $this->get_current_feed();
            echo "<script type=text/javascript> var CWFeedAddonSettings = " . json_encode($feed) . "</script>";

            ob_start();
            include( ABSPATH . '/wp-content/plugins/gravitywise/public/js/vue/feed-builder/feed-builder.php' );
            $injectedDiv = ob_get_clean();

            $base_section = array(
                'description' => ($injectedDiv),
	            'fields' => array(array(
	            ))
            );
            echo "<script type=text/javascript> var CWFeedAddonSettings = " . json_encode($feed) . "</script>";

            // array_unshift($other_sections, $base_section); // adding the base field to be the first in the list
            return array($base_section);
        }


        /**
         * Define the columns on the feeds list
         * This will show the feed name, and a list of all the Actions defined for this feed.
         *
         * @return array
         */
        public function feed_list_columns()
        {
            return array(
                'name'      => __('Name', 'gravitywise'),
            );
        }

        public function feed_retrial_callback()
        {
            include( ABSPATH . '/wp-content/plugins/gravitywise/public/js/vue/retry/retry-app.php' );
        }

        #region Scripts and Styles

        /**
         * Overrides the scripts function to add custom javascript
         * 
         * @return array of all scripts enqueued 
         */
        public function scripts()
        {
            $feed_srcPath = plugins_url('/../public/js/common/feed-script.js', __FILE__);
            $editor_srcPath = plugins_url('/../public/js/common/editor.js', __FILE__);

            $scripts = array(
                array(
                    'handle'  => 'feed_script_js',
                    'src'     => $feed_srcPath,
                    'version' => $this->_version,
                    'deps'    => array('jquery'),
                    'enqueue' => array(
                        array('admin_page' => array('form_settings'))
                    ),
                ),
                array(
                    'handle'  => 'editor_js',
                    'src'     => $editor_srcPath,
                    'version' => $this->_version,
                    'deps'    => array('jquery'),
                    'enqueue' => array(
                        array('admin_page' => array('form_settings'))
                    ),
                ),
            );

            return array_merge(parent::scripts(), $scripts);
        }

        /**
         * Overrides the styles function to add custom style sheet
         * 
         * @return array of all styles enqueued 
         */
        public function styles()
        {
            $srcPath = plugins_url('/../public/css/feed_styles.css', __FILE__);
            $styles = array(
                array(
                    'handle'  => 'feed_styles_css',
                    'src'     => $srcPath,
                    'version' => $this->_version,
                    'enqueue' => array(
                        array('admin_page' => array('form_settings', 'feed_retrials')),
                    )
                )
            );

            return array_merge(parent::styles(), $styles);
        }
    }
}
