<?php
/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/

require_once ABSPATH . "/wp-content/plugins/gravitywise/includes/common/common-variables.php";

class bizz_Helper
{
    /**
     * Retrieves the time zones
     *
     * @return array a list of time zones
     */
    public static function get_zones()
    {
        $timezones = DateTimeZone::listIdentifiers();
        $zones_array = array();
        $item  = array(
            "label" => ' ',
            "value" => ''
        );
        array_push($zones_array, $item);
        $timestamp = time();
        foreach ($timezones as $zone) {
            date_default_timezone_set($zone);

            $item  = array(
                "label" => 'UTC/GMT ' . date('P', $timestamp) . ' - ' . str_replace('_', ' ', $zone),
                "value" => 'UTC/GMT ' . date('P', $timestamp) . ' - ' . str_replace('_', ' ', $zone)
            );
            array_push($zones_array, $item);
        }

        return $zones_array;
    }

    /**
     * Callback validation for settings page input
     *
     * @param string input
     *
     * @return bool True, if input has value. Otherwise, false.
     */
    public static function isValidInput($input) {
        if (empty($input)) {
            return false;
        }
        return true;
    }

    /**
     * Retrieves the api versions in store
     *
     * @return array list of api versions
     */
    public static function getVersionChoices()
    {
        $apiDir = 
        $getFileNames = scandir(ABSPATH . 'wp-content/plugins/gravitywise/config/open-api-documentation');

        $versions = array();

        $item  = array(
            "label" => '',
            "value" => ''
        );
        array_push($versions, $item);

        foreach ($getFileNames as $fileName) {
            if ($fileName !== bizz_CommonVariables::$fileSeparator && $fileName !== bizz_CommonVariables::$folderSeparator) {
				$nFile = preg_replace('/(_Basic|_Premium|_Pro|_Free)\.JSON/','',$fileName);

            	$item = array (
                    "label" => $nFile,
                    "value" => $nFile
                );
            	if(!in_array($item,$versions)){
                    array_push($versions, $item);
	            }
            }
        }
        return $versions;
    }
}
