<?php

/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/
require_once dirname( __FILE__, 6 ) . '/wp-load.php';
class bizz_OpenAPIParser
{
    public static function getOpenAPIContents()
    {
        $settings = get_option( 'gravityformsaddon_gravitywise_settings' );
        $file_name = $settings['cw_version'];
        $file_path = ABSPATH . 'wp-content/plugins/gravitywise/config/open-api-documentation/' . $file_name . '_Free.JSON';
        $filesize = filesize( $file_path );
        $filesizeLimit = 1500000;
        if ( $filesize > $filesizeLimit ) {
            throw new Exception( 'File exceeds maximum allowed size' );
        }
        $open_api_doc = file_get_contents( $file_path, true );
        return json_decode( $open_api_doc, true );
    }
    
    public static function selectfromSchema( $id )
    {
        $contents = bizz_OpenAPIParser::getOpenAPIContents();
        foreach ( $contents as $key => $value ) {
            if ( is_array( $value ) && $key === $id ) {
                return $value;
            }
        }
    }
    
    public static function getCwPaths()
    {
        return bizz_OpenAPIParser::selectfromSchema( 'paths' );
    }
    
    public static function getCwComponentsFull()
    {
        return bizz_OpenAPIParser::selectFromSchema( 'components' )['schemas'];
    }
    
    public static function getCWFields( $component )
    {
        return bizz_OpenAPIParser::getCwComponentsFull()[$component]['properties'];
    }
    
    public static function getCWRequired( $component )
    {
        if ( isset( bizz_OpenAPIParser::getCwComponentsFull()[$component]['required'] ) ) {
            return bizz_OpenAPIParser::getCwComponentsFull()[$component]['required'];
        }
        return [];
    }
    
    public static function getBuilderProperties( $component )
    {
        $res = [];
        $fields = bizz_OpenAPIParser::getCWFields( $component );
        foreach ( $fields as $key => $value ) {
            
            if ( isset( $value['$ref'] ) ) {
                $res[$key] = bizz_OpenAPIParser::getBuilderProperties( substr( $value['$ref'], 21 ) );
            } else {
                
                if ( $key == '_info' ) {
                    continue;
                } else {
                    
                    if ( $value['type'] == 'array' ) {
                        $res[$key] = [];
                        $res[$key][] = bizz_OpenAPIParser::getBuilderProperties( substr( $value['items']['$ref'], 21 ) );
                    } else {
                        $res[$key] = $value['type'];
                    }
                
                }
            
            }
        
        }
        return $res;
    }
    
    public static function getCWProperties( $component )
    {
        $res = array(
            'condition' => [],
            'child'     => [],
            'custom'    => [],
        );
        $fields = bizz_OpenAPIParser::getCWFields( $component );
        foreach ( $fields as $key => $value ) {
            
            if ( $key === 'customFields' ) {
                $custom = bizz_OpenAPIParser::getCWFields( 'CustomFieldValue' );
                foreach ( $custom as $custom_key => $custom_value ) {
                    $res['custom'][$custom_key] = $custom_value['type'];
                }
                continue;
            }
            
            bizz_OpenAPIParser::getCondition( $res['condition'], $key, $value );
            
            if ( $value['type'] == 'array' ) {
                $children = bizz_OpenAPIParser::getCWFields( substr( $value['items']['$ref'], 21 ) );
                foreach ( $children as $child_key => $child_value ) {
                    bizz_OpenAPIParser::getCondition(
                        $res['child'],
                        $child_key,
                        $child_value,
                        $key . '.*.'
                    );
                }
            }
        
        }
        return $res;
    }
    
    private static function getCondition(
        &$arr,
        $key,
        $value,
        $prepend = ""
    )
    {
        
        if ( isset( $value['$ref'] ) ) {
            $ref_props = bizz_OpenAPIParser::getCWFields( substr( $value['$ref'], 21 ) );
            foreach ( $ref_props as $ref_key => $ref_val ) {
                if ( $ref_val['type'] != null ) {
                    $arr[$prepend . $key . '.' . $ref_key] = $ref_val['type'];
                }
            }
        } elseif ( $value['type'] !== 'array' ) {
            $arr[$prepend . $key] = $value['type'];
        }
    
    }
    
    public static function getCwComponents( $show_all, $http_method )
    {
        $paths = bizz_OpenAPIParser::getCwPaths();
        $components = array();
        foreach ( $paths as $endpoint => $endpoint_type ) {
            try {
                $component = substr( reset( $endpoint_type[$http_method]['responses']['200']['content'] )['schema']['items']['$ref'], 21 );
            } catch ( Exception $e ) {
                continue;
            }
            $matches = [];
            preg_match_all( '/[A-Z][a-z]+/', $component, $matches );
            if ( count( $matches[0] ) == 1 || $show_all === 'true' ) {
                $components[$component] = $endpoint;
            }
        }
        ksort( $components );
        return $components;
    }
    
    public static function verifyAPIConfig()
    {
        $settings = get_option( 'gravityformsaddon_gravitywise_settings' );
        return !empty($settings['cw_site']) && !empty($settings['company_id']);
    }

}