<?php
/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/

/**
 * Class bizz_ActionMapper
 */
class bizz_ActionMapper
{
    
    public static function init_table(){
        global $wpdb;
        $wpdb->query(
            $wpdb->prepare(
                "CREATE TABLE `{$wpdb->prefix}cw_action_attempt` (
					  `id` mediumint(9) NOT NULL AUTO_INCREMENT,
					  `entry_attempt_id` mediumint(9) NOT NULL,
					  `meta_key` mediumint(9) NOT NULL,
					  `meta` longtext,
					  `value` longtext,
					  `status` enum('Scrubbed','Fail','Success','No Connection','Reverted','Duplicate','Dismissed','Wait') DEFAULT NULL,
					  `attempt_count` mediumint(9) DEFAULT NULL,
					  `error_code` longtext,
					  `has_merged` tinyint(1) DEFAULT NULL,
					  `merge_id` mediumint(9) DEFAULT NULL,
					  PRIMARY KEY (`id`)
					)",
	            ''
            )
        );
    }

    private static function get_changed()
    {
        global $wpdb;
        $id = $wpdb->insert_id;
        return $wpdb->get_results(
            $wpdb->prepare(
                "SELECT
                    *
                FROM
                    {$wpdb->prefix}cw_action_attempt
                WHERE
                    id = %d
                ",
                $id
            )
        );
    }

    public static function get_action_attempts($entry_id)
    {
        global $wpdb;
        $actions = $wpdb->get_results(
            $wpdb->prepare(
                "SELECT
                            A.*
                        FROM
                            {$wpdb->prefix}cw_action_attempt A
                            LEFT JOIN {$wpdb->prefix}cw_entry_attempt B ON A.entry_attempt_id = B.id
                        WHERE
                            B.id = %d
                ",
                $entry_id
            )
        );

        $res = [];
        foreach ($actions as $k => $a) {
            $new_action = new bizz_FeedAction($a->id,
	            json_decode($a->meta,true),
	            json_decode($a->value,true),
	            $a->status,
	            json_decode($a->error_code)??(object)[],
                $a->merge_id ?? 0,
                $a->has_merged ?? 0
            );
            array_push($res, $new_action);
        }
        return $res;
    }

    public static function get_action_attempt($id){
        global $wpdb;
        return $wpdb->get_results(
            $wpdb->prepare(
                "SELECT
                    *
                FROM 
                    {$wpdb->prefix}cw_action_attempt
                WHERE 
                    id = %d",
                $id
            )
        );
    }

    public static function update_action_attempt(bizz_FeedAction $attempt)
    {
        global $wpdb;
        $wpdb->query(
            $wpdb->prepare(
                "UPDATE 
                    {$wpdb->prefix}cw_action_attempt
                SET 
                    attempt_count = attempt_count+1,
                    error_code = %s,
                    status = %s,
                    value = %s,
                    meta = %s,
                    merge_id = %d,
					has_merged = %d
                WHERE 
                    id = %d;
                ",
                json_encode($attempt->error_code),
                $attempt->status,
                json_encode($attempt->value),
                json_encode($attempt->meta),
                $attempt->merge_id,
                $attempt->has_merged,
                $attempt->id
            )
        );
    }

    public static function create_action_attempt($entry_attempt_id, $meta_key, $meta, $value, $status)
    {
        global $wpdb;
        $wpdb->query(
            $wpdb->prepare(
                "INSERT INTO 
                    {$wpdb->prefix}cw_action_attempt(
                        entry_attempt_id,
                        meta_key,
                        meta,
                        value,
                        status,
                        error_code,
                        attempt_count
                    ) 
                    VALUES 
                    (
                        %d,
                        %d,
                        %s,
                        %s,
                        %s,
                        '',
                        1   
                    );
                ",
                $entry_attempt_id,
                $meta_key,
                json_encode($meta),
                json_encode($value),
                $status
            )
        );
        $a = bizz_ActionMapper::get_changed()[0];
        return new bizz_FeedAction($a->id,json_decode($a->meta,true),json_decode($a->value,true),$a->status,(object)json_decode($a->error_code));
    }

    public static function delete_action_attempt($attempt)
    {
        global $wpdb;
        $wpdb->query(
            $wpdb->prepare(
                "DELETE 
                        FROM {$wpdb->prefix}cw_action_attempt 
                    WHERE 
                        id = %d;",
                $attempt->id
            )
        );
    }

    public static function scrub_action_attempt($attempt)
    {
        global $wpdb;
        $wpdb->query(
            $wpdb->prepare(
                "UPDATE {$wpdb->prefix}cw_action_attempt
                    SET 
                        meta = '',
                        error_code = '',
                        value = '',
                        status = 'Scrubbed',
                        attempt_count = ''
                    WHERE 
                        id = %d;",
                $attempt->id
            )
        );
    }
}
