<?php
/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/

require_once ABSPATH . '/wp-content/plugins/gravitywise/includes/common/open-api-parser.php';

/**
 * Class bizz_DuplicateMatch
 */
class bizz_DuplicateMatch {

	/**
	 * @param bizz_ConnectwiseClient $cw_api
	 * @param string $endpoint
	 * @param stdClass $request
	 * @param int|null $duplicate_id
	 *
	 * @return bool
	 */
	public static function detect_duplicate( bizz_ConnectwiseClient $cw_api, string $endpoint, stdClass $request, int &$duplicate_id = null ): bool {
		$duplicate = false;
		$match     = [];

		if ( $endpoint == '/company/companies' ) {
			$hard_match = false;
			if ( isset( $request->phoneNumber ) ) {
				$hard_match = $hard_match || bizz_DuplicateMatch::perfect_match( $cw_api, $endpoint, 'phoneNumber', $request->phoneNumber, $match );
			}
			if ( isset( $request->faxNumber ) ) {
				$hard_match = $hard_match || bizz_DuplicateMatch::perfect_match( $cw_api, $endpoint, 'faxNumber', $request->faxNumber, $match );
			}
			if ( isset( $request->website ) ) {
				$hard_match = $hard_match || bizz_DuplicateMatch::perfect_match( $cw_api, $endpoint, 'website', $request->website, $match );
			}

			if ( $hard_match ) {
				$duplicate = $duplicate || bizz_DuplicateMatch::check_similar( $match, (array) $request, 75 );
			} else {
				$duplicate = $duplicate || bizz_DuplicateMatch::company_city_match( $cw_api, (array) $request, 90 );
			}
		} else if ( $endpoint == '/company/contacts' ) {
			if ( isset( $request->defaultPhoneNbr ) ) {
				$duplicate = $duplicate || bizz_DuplicateMatch::perfect_match( $cw_api, $endpoint, 'defaultPhoneNbr', $request->defaultPhoneNbr );
			}
			if ( isset( $request->communicationItems ) ) {
				foreach ( $request->communicationItems as $value ) {
					if ( isset( $value->value ) ) {
						$duplicate = $duplicate || bizz_DuplicateMatch::communication_items_match( $cw_api, '/company/contacts', $value->type->name, $value->value );
					}
				}
			}
			//$duplicate = $duplicate || bizz_DuplicateMatch::contact_name_match($cw_api,$request);
		} else {
			//TODO replace with component name lookup

			if ( isset( $request->name ) ) {
				$duplicate = $duplicate || bizz_DuplicateMatch::perfect_match( $cw_api, $endpoint, 'name', $request->name, $match );
			} else if ( isset( $request->description ) ) {
				$duplicate = $duplicate || bizz_DuplicateMatch::perfect_match( $cw_api, $endpoint, 'description', $request->description, $match );
			} else if ( isset( $request->title ) ) {
				$duplicate = $duplicate || bizz_DuplicateMatch::perfect_match( $cw_api, $endpoint, 'title', $request->title, $match );
			} else if ( isset( $request->summary ) ) {
				$duplicate = $duplicate || bizz_DuplicateMatch::perfect_match( $cw_api, $endpoint, 'summary', $request->summary, $match );
			}
		}

		if ( $duplicate && count( $match ) ) {
			$duplicate_id = $match[0]->id;
		}

		return $duplicate;
	}

	/**
	 * @param bizz_ConnectwiseClient $cw_api
	 * @param string $extension
	 * @param string $property
	 * @param string $value
	 * @param array $match
	 *
	 * @return bool
	 */
	//TODO FIX MATCHES
	public static function perfect_match( bizz_ConnectwiseClient $cw_api, string $extension, string $property, string $value, array &$match = [] ): bool {
		$response = $cw_api->getCwData( $extension, 'conditions=' . urlencode( "$property=\"$value\"" ) );
		if ( $response != null ) {
			$match = $response;
		}

		return $response != null;
	}

	/**
	 * @param array $entities
	 * @param array $value
	 * @param float $threshold
	 *
	 * @return bool
	 */
	public static function check_similar( array $entities, array $value, float $threshold ): bool {
		foreach ( $entities as $entity ) {
			similar_text( strtolower( $value['name'] ), strtolower( $entity->name ), $percent_similar );
			if ( $percent_similar > $threshold ) {
				return true;
			}
		}

		return false;
	}

	/**
	 * @param bizz_ConnectwiseClient $cw_api
	 * @param array $value
	 * @param float $threshold
	 *
	 * @return bool
	 */
	public static function company_city_match( bizz_ConnectwiseClient $cw_api, array $value, float $threshold ): bool {
		$conditions = '';
		if ( isset( $value['city'] ) && isset( $value['state'] ) ) {
			$city       = $value['city'];
			$state      = $value['state'];
			$conditions = "(city = \"$city\" and state = \"$state\")";
		} else if ( isset( $value['state'] ) ) {
			$state      = $value['state'];
			$conditions = "(state = \"$state\")";
		}
		if ( isset( $value['zip'] ) ) {
			$zipcode = $value['zip'];
			if ( $conditions != '' ) {
				$conditions .= ' or ';
			}
			$conditions .= "(zip = \"$zipcode\")";
		}

		$data = $cw_api->getCwData( '/company/companies', 'conditions=' . urlencode( $conditions ) . '&pagesize=1000' );

		return bizz_DuplicateMatch::check_similar( $data, $value, $threshold );
	}

	/**
	 * @param bizz_ConnectwiseClient $cw_api
	 * @param string $extension
	 * @param string $property
	 * @param string $value
	 * @param int $match
	 *
	 * @return bool
	 */
	public static function communication_items_match( bizz_ConnectwiseClient $cw_api, string $extension, string $property, string $value, int &$match = 0 ): bool {
		$response = $cw_api->getCwData( $extension, 'childConditions=' . urlencode( "communicationItems/value=\"$value\" AND communicationItems/communicationType=\"$property\"" ) );
		if ( $response != null ) {
			$match = $response[0]->id;
		}

		return $response != null;
	}

	/**
	 * @param bizz_ConnectwiseClient $cw_api
	 * @param string $extension
	 * @param string $property
	 * @param                   $domain
	 *
	 * @return bool
	 */
	public static function domain_match( bizz_ConnectwiseClient $cw_api, string $extension, string $property, $domain ): bool {
		preg_match( '/.*?\.(.*?\..*)/', $domain[ $property ], $matches );


		$response = $cw_api->getCwData( $extension, 'conditions=' . urlencode( "$property like \"%.$matches[1]\"" ) );

		return $response != null;
	}

	/**
	 * @param bizz_ConnectwiseClient $cw_api
	 * @param array $value
	 *
	 * @return bool
	 */
	public static function contact_name_match( bizz_ConnectwiseClient $cw_api, array $value ): bool {
		$city     = $value['city'];
		$state    = $value['state'];
		$zip_code = $value['zip'];

		$response = $cw_api->getCwData( '/company/contacts', 'conditions=' . urlencode( "city = \"$city\" or state = \"$state\" or zip =  \"$zip_code\")" ) . '&pagesize=1000' );

		foreach ( $response as $contact ) {
			if ( strtolower( $contact->firstName ) == strtolower( $value['firstName'] ) && strtolower( $contact->lastName ) == strtolower( $value['lastName'] ) ) {
				return true;
			}
		}

		return false;
	}
}
