<?php
/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/

require_once ABSPATH . '/wp-content/plugins/gravitywise/includes/feeds/feed-entry.php';

/**
 * Class bizz_EntryMapper
 */
class bizz_EntryMapper
{

    public static function init_table(){
        global $wpdb;
        $wpdb->query(
            $wpdb->prepare(
                "CREATE TABLE `{$wpdb->prefix}cw_entry_attempt` (
					  `id` mediumint(9) NOT NULL AUTO_INCREMENT,
					  `feed_id` mediumint(9) NOT NULL,
					  `entry_id` mediumint(9) NOT NULL,
					  `condition_meta` longtext,
					  `dismissed` tinyint(1) DEFAULT NULL,
					  `condition_validated` enum('Not Checked','True','False') DEFAULT 'Not Checked',
					  PRIMARY KEY (`id`)
					)", ''
            )
        );
    }

	/**
	 * @return array
	 */
	private static function get_changed():array
    {
        global $wpdb;
        $insert_id = $wpdb->insert_id;
        return $wpdb->get_results(
            $wpdb->prepare(
                "SELECT
                    *
                FROM
                    {$wpdb->prefix}cw_entry_attempt
                WHERE
                    id = %d",
                $insert_id
            )
        );
    }

	/**
	 * @param int $entry_id
	 *
	 * @return bizz_FeedEntry
	 */
	public static function get_entry_attempt(int $entry_id): bizz_FeedEntry {
        global $wpdb;
        $attempt = $wpdb->get_results(
            $wpdb->prepare(
                "SELECT
                    *
                FROM
                    {$wpdb->prefix}cw_entry_attempt
                WHERE
                    entry_id = %d",
                $entry_id
            )
        );
        $attempt = $attempt[0];
        return new bizz_FeedEntry($attempt->id, $attempt->feed_id, $attempt->entry_id, json_decode($attempt->condition_meta),$attempt->dismissed,$attempt->condition_validated);
    }

	/**
	 * @param int $entry_id
	 *
	 * @return bizz_FeedEntry
	 */
	public static function get_entry_attempt_id(int $entry_id): bizz_FeedEntry {
        global $wpdb;
        $attempt = $wpdb->get_results(
            $wpdb->prepare(
                "SELECT
                    *
                FROM
                    {$wpdb->prefix}cw_entry_attempt
                WHERE
                    id = %d",
                $entry_id
            )
        );
        $attempt = $attempt[0];
        return new bizz_FeedEntry($attempt->id, $attempt->feed_id, $attempt->entry_id, json_decode($attempt->condition_meta),$attempt->dismissed,$attempt->condition_validated);
    }

	/**
	 * @param stdClass $entry
	 *
	 * @return bizz_FeedEntry
	 */
	public static function create_entry_attempt(stdClass $entry): bizz_FeedEntry {
        global $wpdb;
        $wpdb->query(
            $wpdb->prepare(
                "
                INSERT INTO {$wpdb->prefix}cw_entry_attempt(
                feed_id,
                entry_id,
                condition_meta,
                dismissed
                ) VALUES (
                %d,
                %d,
                %s,
                %d
                );
            ",
                $entry->feed_id,
                $entry->entry_id,
                json_encode($entry->condition_meta),
                $entry->dismissed ?? 0
            )
        );
        $result = bizz_EntryMapper::get_changed()[0];
        return new bizz_FeedEntry($result->id,
	        $result->feed_id,
	        $result->entry_id,
	        (object)$result->condition_meta,
	        $result->dismissed,
	        $result->condition_validated);
    }

	/**
	 * @param bizz_FeedEntry $entry
	 */
	public static function update_entry_attempt(bizz_FeedEntry $entry)
    {
        global $wpdb;
        $wpdb->query($wpdb->prepare(
            "UPDATE
                {$wpdb->prefix}cw_entry_attempt
            SET
                feed_id = %d,
                entry_id = %d,
                condition_meta = %s,
                dismissed = %d
            WHERE
                id = %d
            ",
            $entry->feed_id,
            $entry->entry_id,
            json_encode($entry->condition_meta),
            $entry->dismissed,
            $entry->id
        ));
    }

	/**
	 * @param bizz_FeedEntry $entry
	 */
	public static function set_check_condition(bizz_FeedEntry $entry){
        global $wpdb;
        $wpdb->query(
            $wpdb->prepare(
                "UPDATE {$wpdb->prefix}cw_entry_attempt
                SET
                    condition_validated = %s
                WHERE
                    id = %d",
                $entry->condition_validated,
                $entry->id
            )
        );
    }

	/**
	 * @param bizz_FeedEntry $entry
	 */
	public static function delete_entry_attempt(bizz_FeedEntry $entry)
    {
        global $wpdb;
        $wpdb->query(
            $wpdb->prepare(
                "DELETE 
                    FROM {$wpdb->prefix}cw_entry_attempt
                WHERE 
                    id = %d
                ",
                $entry->id
            )
        );
    }

	/**
	 * @param bizz_FeedEntry $entry
	 */
	public static function scrub_entry_attempt(bizz_FeedEntry $entry)
    {
        global $wpdb;
        $wpdb->query(
            $wpdb->prepare(
                "UPDATE {$wpdb->prefix}cw_entry_attempt
                SET
                    condition_meta = ''
                WHERE
                    id = %d
                ",
                $entry->id
            )
        );
    }

	/**
	 * @param int $entry_id
	 *
	 * @return array
	 */
	public static function get_form_data(int $entry_id): array {
        global $wpdb;
        $entry_values = $wpdb->get_results(
            $wpdb->prepare(
            "SELECT
                A.meta_key,
                A.meta_value
            FROM
                {$wpdb->prefix}gf_entry_meta A
                LEFT JOIN {$wpdb->prefix}cw_entry_attempt B on A.entry_id = B.entry_id
            WHERE
                B.entry_id = %d;    
            ",$entry_id)
        );

        $form_data = [];
        foreach ($entry_values as $value) {
            $form_data[$value->meta_key]['value'] = $value->meta_value;
        }

        return $form_data;
    }

	/**
	 * @param int $entry_id
	 *
	 * @return array
	 */
	public static function get_entry_values(int $entry_id): Array
    {
        global $wpdb;
        return $wpdb->get_results(
            $wpdb->prepare(
                "SELECT meta_key,meta_value FROM {$wpdb->prefix}gf_entry_meta WHERE entry_id = %d;
            ",
                $entry_id
            )
        );
    }

	/**
	 * @param int $feed_id
	 *
	 * @return stdClass
	 */
	public static function get_feed_meta(int $feed_id): array {
        global $wpdb;
        return json_decode($wpdb->get_results(
            $wpdb->prepare(
                "SELECT meta FROM {$wpdb->prefix}gf_addon_feed WHERE id = %d",
                $feed_id
            )
        )[0]->meta,true);
    }
}
