<?php
/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/
	require_once ABSPATH . '/wp-content/plugins/gravitywise/includes/feeds/duplicate-match.php';

/**
 * Class bizz_FeedAction
 */
class bizz_FeedAction {
	public $id;
	public $meta;
	public $value;
	public $status;
	public $error_code;
	public $mt;
	public $merge_id;
	public $has_merged;

	/**
	 * bizz_FeedAction constructor.
	 *
	 * @param int      $id
	 * @param stdClass $meta
	 * @param stdClass $value
	 * @param string   $status
	 * @param stdClass $error_code
	 * @param int      $merge_id
	 * @param int      $has_merged
	 */
	public function __construct( int $id, array $meta, array $value, string $status, stdClass $error_code, int $merge_id = 0, int $has_merged = 0 ) {
		$this->id         = $id;
		$this->meta       = $meta;
		$this->value      = $value;
		$this->status     = $status;
		$this->error_code = $error_code;
		$this->merge_id   = $merge_id;
		$this->has_merged = $has_merged;
	}

	/**
	 * @param bizz_MergeTags $mt
	 */
	public function set_mt( bizz_MergeTags $mt ) {
		$this->mt = $mt;
	}

	public function apply_catchall(&$request){
		$conf = (object) get_option('gravityformsaddon_gravitywise_settings');


		$catchall_application_list=['Activity','Opportunity','Ticket','Contact'];
		if(in_array($this->meta['component']['name'],$catchall_application_list)){
			if(!array_key_exists('company',$request)){
				if($conf->catchall_id!=''){
					$request['company']['id'] = $conf->catchall_id;
				}else if($conf->catchall_name!=''){
					$request['company']['name'] = $conf->catchall_name;
				}
			}
		}
	}

	/**
	 * @param bool $processMT
	 * @return object
	 */
	public function build_request( bool $processMT ): object {
		if($processMT) {
			array_walk_recursive($this->value,function(&$item,$key){
				$item = urldecode( $this->mt->replaceMergeTags( urlencode( $item ) ) );
			});
		}
		$this->apply_catchall($this->value);
		return (object) $this->value;
	}

	/**
	 * @param stdClass $request
	 * @param bizz_ConnectwiseClient $cw_api
	 *
	 * @return bool
	 */
	public function execute_request( stdClass $request, bizz_ConnectwiseClient $cw_api, bool $bypass_duplicate = false): bool {
		if(!$bypass_duplicate) {
			if ( isset( $this->meta['duplicatePolicy'] ) ) {
				$matching = $this->meta['duplicatePolicy'];
				if ( $matching != 'none' ) {
					$duplicate = bizz_DuplicateMatch::detect_duplicate( $cw_api, $this->meta['component']['endpoint'], $request, $out );
					if ( $duplicate ) {
						$this->merge_id   = $out;
						$this->has_merged = 0;
						if ( $matching == 'cautious' ) {
							$this->status = 'Duplicate';
                            bizz_ActionMapper::update_action_attempt( $this );
							try{
								$this->mail($this->meta['component']['name']);
							}catch(Exception $error){}
							return false;
						}
						$response         = $this->patch_request( $request, $out, $cw_api );
						$this->has_merged = 1;
					}
				}
			}
		}

		if($this->meta['component']['endpoint'] === '/sales/activities'){
			$request = $this->check_round_robin($request,$cw_api);
		}

		if(!isset($response)){
			$response = $cw_api->postCwData( $this->meta['component']['endpoint'], $request );
		}

		return $this->finish_execution($response);

	}

	public function check_round_robin( $request, bizz_ConnectwiseClient $cw_api){
		$found = false;

		if(mb_substr($request->assignTo['id'],0,1) === '⌻'){
			$found = true;
		}

		if($found===false){
			return $request;
		}

		preg_match('/⌻(\d+)⌻/', $request->assignTo['id'], $matches);


		$teamId = intval($matches[1]);
		$response = $cw_api->getCwData("/sales/salesTeams/$teamId/members",'fields=member');
		$members = [];
		foreach ($response as $person){
			array_push($members,$person->member->id);
		}

		$new_id = 0;
		$member_string = join(',',$members);
		if(isset($request->opportunity['id'])){
			$x = $request->opportunity['id'];
			$discovered = $cw_api->getCwData('/sales/opportunities',
				"conditions=primarySalesRep/id%20in%20($member_string)%20and%20id=$x");
			if(count($discovered)!=0){
				$new_id = $discovered[0]->primarySalesRep->id;
			}
		}


		if($new_id!=0){
			$request->assignTo['id'] = $new_id;
		}else{
			$request->assignTo['id'] = preg_replace('/⌻\d+⌻/','',$request->assignTo['id']);
		}
		return $request;
	}

	public function finish_execution($response){
		if ( $this->mt != null ) {
			$this->add_response_mt( $response );
		}

		if ( isset( $response->errors ) || isset( $response->code ) ) {
			if(!isset($response->errors)){
				$this->status = 'Fail';
			}else if ( gettype( $response->errors ) == 'string' && strpos( $response->errors, 'Curl error: Resolving timed out' ) !== false ) {
				$this->status = 'No Connection';
			} else {
				$this->status = 'Fail';
			}
		} else {
			$this->status = 'Success';
		}

		$this->error_code = $response;
        bizz_ActionMapper::update_action_attempt( $this );

		return $this->status == 'Success';
	}


	public function execute_merge(stdClass $request, bizz_ConnectwiseClient $cw_api, bool $merge){
		$this->has_merged = 1;
		if($merge){
			$response         = $this->patch_request( $request, $this->merge_id, $cw_api );
			$this->finish_execution($response);
		}else{
			$this->execute_request($request,$cw_api,true);
		}
	}

	/**
	 * @param stdClass          $request
	 * @param int               $target
	 * @param bizz_ConnectwiseClient $cw_api
	 */
	public function patch_request( stdClass $request, int $target, bizz_ConnectwiseClient $cw_api )
	{
		$new_request = [];
		foreach ( $request as $property => $value ) {
			array_push( $new_request,
				(object) [ 'op' => 'replace', 'path' => $property, 'value' => $value ] );
		}
		$asJson = json_encode($new_request);
		return $cw_api->patchCwData( $this->meta['component']['endpoint'] . '/' . $target, $new_request);
	}

	/**
	 * @param stdClass $response
	 */
	public function add_response_mt( stdClass $response ) {
		$action_responses = $this->mt->getActionResponses();
		array_push( $action_responses, json_encode( $response ) );
		$this->mt->setActionResponses( $action_responses );
	}

	private function mail($component) {
		$settings = get_option('gravityformsaddon_gravitywise_settings');
		wp_mail($settings['approval_email'],
			'Duplicate '.$component.' Waiting For Approval',
			'A Duplicate'.$component.'was found. Visit '. get_site_url . '/wp-admin/admin.php?page=feed_retrials to review the match');


	}
}
