<?php
/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/

require_once ABSPATH . '/wp-load.php';
require_once ABSPATH . '/wp-content/plugins/gravitywise/includes/feeds/entry-mapper.php';
require_once ABSPATH . '/wp-content/plugins/gravitywise/includes/common/merge-tags.php';

/**
 * Class bizz_FeedDispatcher
 */
class bizz_FeedDispatcher
{
    public $entries;
    public $cw_api;

	/**
	 * bizz_FeedDispatcher constructor.
	 *
	 * @param bizz_ConnectwiseClient $cw_api
	 */
	public function __construct(bizz_ConnectwiseClient $cw_api)
    {
        $this->entries = [];
        $this->cw_api = $cw_api;
    }

    public function generate_new_entries()
    {
        global $wpdb;
        $unpaired_entries = $wpdb->get_results(
            "SELECT C.id as feed_id, X.id as entry_id, C.meta as meta, X.date_created as created FROM (
                    SELECT A.id,A.form_id,A.date_created FROM {$wpdb->prefix}gf_entry A 
                        LEFT JOIN {$wpdb->prefix}cw_entry_attempt B ON A.id=B.entry_id
                        WHERE B.entry_id IS NULL)
                as X INNER JOIN {$wpdb->prefix}gf_addon_feed C ON X.form_id=C.form_id WHERE C.addon_slug = 'gravitywise'"
        );
        foreach ($unpaired_entries as $value) {
            $meta = json_decode($value->meta);
            if(!isset($meta->created_date) || strtotime($meta->created_date) < strtotime($value->created)){
	            $condition = $meta->CWFeedSettings->conditions;
	            $value->condition_meta = $condition;
	            $entry = bizz_EntryMapper::create_entry_attempt($value);
	            $entry->generate_actions();
	            array_push($this->entries, $entry);
            }
        }
    }

	/**
	 * @param bool $fetch_actions
	 */
	public function process_entries(bool $fetch_actions=false)
    {
	    /** @var bizz_FeedEntry $value */
	    foreach ($this->entries as $value) {
            if($fetch_actions){
                $value->fetch_actions();
            }

            
            $value->set_merge_tag();
            $value->process_actions($this->cw_api);
        }
    }
}
