<?php
/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/

require_once ABSPATH . '/wp-content/plugins/gravitywise/includes/feeds/feed-action.php';
require_once ABSPATH . '/wp-content/plugins/gravitywise/includes/feeds/action-mapper.php';

/**
 * Class bizz_FeedEntry
 */
class bizz_FeedEntry
{
    public $id;
    public $feed_id;
    public $entry_id;
    public $condition_meta;
    public $actions;
    public $mt;
    public $dismissed;
    public $condition_validated;

	/**
	 * bizz_FeedEntry constructor.
	 *
	 * @param int      $id
	 * @param int      $feed_id
	 * @param int      $entry_id
	 * @param stdClass $condition_meta
	 * @param int      $dismissed
	 * @param string   $condition_validated
	 */
	public function __construct(int $id, int $feed_id, int $entry_id, stdClass $condition_meta, int $dismissed, string $condition_validated)
    {
        $this->id = $id;
        $this->feed_id = $feed_id;
        $this->entry_id = $entry_id;
        $this->condition_meta = $condition_meta;
        $this->dismissed = $dismissed;
        $this->condition_validated = $condition_validated;
        $this->actions = [];
    }

    public function fetch_actions()
    {
        $this->actions = bizz_ActionMapper::get_action_attempts($this->id);
    }

    public function set_merge_tag()
    {
        $this->mt = new bizz_MergeTags(bizz_EntryMapper::get_form_data($this->entry_id));
    }

	/**
	 * @return bool
	 */
	public function validate_conditional(): bool {

		if (isset($this->condition_meta->scalar)){
            $conditions = json_decode($this->condition_meta->scalar);

		}else{
			$conditions = $this->condition_meta;
		}
        if ($conditions->active == false) {
            return true;
        }

		foreach ($conditions->statements as $value) {

	        $left_hand_side = urldecode($this->mt->replaceMergeTags(urlencode($value->property)));
            $right_hand_side = urldecode($this->mt->replaceMergeTags(urlencode($value->value)));
            $result = false;
            switch ($value->operator) {
                case '=':
                    $result = $left_hand_side == $right_hand_side;
                    break;
                case '!=':
                    $result = $left_hand_side != $right_hand_side;
                    break;
                case '<':
                    $result = $left_hand_side < $right_hand_side;
                    break;
                case '>':
                    $result = $left_hand_side > $right_hand_side;
                    break;
            }
            if ($conditions->conjunction == 'ALL' && $result == false) {
                return false;
            } else if ($conditions->conjunction == 'ANY' && $result == true) {
                return true;
            }
        }
        if($conditions->conjunction == 'ALL'){
            return true;
        }
        else {
            return false;
        }
    }

	/**
	 * @param bizz_ConnectwiseClient $cw_api
	 * @param bool              $process_mt
	 */
	public function process_actions(bizz_ConnectwiseClient $cw_api, bool $process_mt = true)
    {
        if($this->validate_conditional()){
            $this->condition_validated = 'True';
            bizz_EntryMapper::set_check_condition($this);
        }else{
            //The condition has not been met, don't run the feed.
            $this->condition_validated = 'False';
            bizz_EntryMapper::set_check_condition($this);
            return;
        }

	    /** @var bizz_FeedAction $value */
	    foreach ($this->actions as $value) {
            $value->set_mt($this->mt);
            if ($value->status == 'Wait') {
                $request = $value->build_request($process_mt);
                if (!$value->execute_request($request, $cw_api)) {
                    break;
                }
            } else if ($value->status == 'Success') {
                $value->add_response_mt($value->error_code);
            } else {
                break;
            }
        }
    }

    public function generate_actions()
    {
        $full_meta = bizz_EntryMapper::get_feed_meta($this->feed_id);
	    foreach ($full_meta['CWFeedSettings']['actions'] as $meta_key => $meta) {
            array_push($this->actions, bizz_ActionMapper::create_action_attempt($this->id, $meta_key, $meta, $meta['request'], 'Wait'));
        }
    }

	/**
	 * @param int $id
	 *
	 * @return bizz_FeedAction
	 */
	public function generate_action(int $id): bizz_FeedAction {
        $meta = bizz_EntryMapper::get_feed_meta($this->feed_id)['CWFeedSettings']['actions'][$id];
        return bizz_ActionMapper::create_action_attempt($this->id, $id, $meta, $meta['request'], 'Wait');
    }
}
