/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/

var cwHelperJs = {

    /**
     * Makes a post request to cwApiHelper.php and returns the results of that request.
     * @param {object} args an object containing any data to be passed along the request.
     * example: {filter: false, apiRequestType: 'get', operation: 'getCWComponents'} - requests that
     * the cwApiHelper make a get request to the CWApi to get the components in CW.
     * @returns {object} the results of the post request.
     */
    AjaxCall: async function(endpoint,queryString = '') {

        // var key = JSON.stringify(args);
        // var cacheItem = window.localStorage.getItem(key);
        // if(cacheItem!=undefined){


        //     console.log("Read from cache");
        //     return JSON.parse(cacheItem);
        // }
        // args.version = storedPhpVariables.cwApiVersion();
        // console.log("Ajax Call Made");
        // var retrievedData = await jQuery.post( {
        //     type: "POST",
        //     url: storedPhpVariables.helperDir(),
        //     data: args,
        //     dataType: "json"
        // })
        var retrievedData = (await fetch(`/wp-content/plugins/gravitywise/includes/api/${endpoint}.php?${queryString}`))
        try {
            retrievedData = retrievedData.json();
        } catch (e) {
            return undefined;
        }


        // window.localStorage.setItem(key,JSON.stringify(retrievedData));

        return retrievedData;
    },

    /**
     * Makes a request to the ConnectWise API helper for a list of names and endpoints for the components.
     * @param {boolean} showAll whether or not to show all components or just the basic ones.
     * @returns {Array} of objects containing a name and an endpoint.
     */
    retrieveConnectWiseComponents: async function(showAll) {

        var res = [];
        //var args = {filter: showAll, apiRequestType: 'get', operation: 'getCWComponents'};
        var data = await this.AjaxCall('common/get-components','all='+showAll);

        if (data == undefined) {
            return;
        }
        jQuery.each(data, function(i, d) {
            res.push({name: i,endpoint: d});
        })
        return res;
    },

    /**
     * Makes a request to the ConnectWise API helper for a list of property names based on a supplied endpoint
     * @param {string} EndPoint the endpoint for which to receive properties.
     * @returns {Array} of property names.
     */
    retrieveConnectWiseProperties: async function(EndPoint) {
        var res = {child: [], condition: [], custom: [], fields: []};
        //var args =  {endpoint: EndPoint, operation: 'getCWComponentProperties'};
        var data = await this.AjaxCall('common/get-properties','component='+EndPoint)
        if(data == undefined){
            return;
        }
        jQuery.each(data.child, function(i, d) {
            res.child.push({name: i, type: d});
        });
        jQuery.each(data.condition, function(i, d) {
            res.condition.push({name: i,type: d});
            if (res.fields.indexOf(i.split('.')[0]) === -1) {
                res.fields.push(i.split('.')[0]);
            }
        });
        jQuery.each(data.custom, function(i, d) {
            res.custom.push({name: i,type: d});
        });
        return res;
    },

    //TODO FIX
    retrieveRequiredFields: async function (Component) {
        var data = await this.AjaxCall('common/get-required', 'component=' + Component)
        var props = await cwHelperJs.retrieveConnectWiseProperties(Component);

        if (data == undefined) {
            return;
        }
        data.forEach(element => {
            for (let i = 0; i < props.condition.length; i++) {
                const prop = props.condition[i];
                if (prop.name == element) {
                    props.condition.splice(i, 1);
                }
            }
        });

        arrayFields = {};
        props.child.forEach(element => {
            if (element.type !== "object") {
                const regex = /(\w+)\.\*\.(\w+)/;
                const res = regex.exec(element.name);
                if (arrayFields[res[1]] === undefined) {
                    arrayFields[res[1]] = {};
                }
                arrayFields[res[1]][res[2]] = element.type;
            }
        });
        if (props.custom.length > 0) {
            arrayFields.custom = {};
            props.custom.forEach(el => {
                arrayFields['custom'][el.name] = el.type;
            });
        }

        return {required: data, properties: props.condition, arrayFields: arrayFields};
    },


    /**
     * Makes a request to the ConnectWise API helper for a list of properties that are required for a supplied endpoint.
     * @param {string} EndPoint the endpoint for which to receive properties.
     * @returns {Array} of property names.
     */
    //TODO FIX
    retrieveRequiredProperties: async function(value, set) {

        var res = [];
        var args = {version: storedPhpVariables.cwApiVersion(), endpoint: value, operation: 'getRequiredProps'};
        var data = await this.AjaxCall(args)
        if (data == undefined) {
            return;
        }
        jQuery.each(data, function (i, d) {
            res.push({propertyName: i, propertyId: d, propertyField: ""});
        });
        if (this.settingsEqual(set, data)) {
            return set;
        } else {
            return res;
        }
    },

    retrieveDatabaseLookup: async function(field, search) {
        var data = await this.AjaxCall('form/get-results', `guid=${cwAddonGuid}&field=${field}&search=${encodeURIComponent(search)}`);
        if (data == undefined) {
            return;
        }
        return data;
    },


    //TODO, make this more elegant.
    settingsEqual: function(settings, data) {
        for (var i = 0; i < settings.length; i++) {
            try {
                if (settings[i].propertyName != data[i].i) {
                    return false;
                }
            } catch {
                return false;
            } finally {
                return true;
            }
        }
    },

    notifyMergeTagChange: async function(field, value) {
        if (typeof cwAddonGuid === 'undefined') {
            return;
        }
        await fetch('/wp-content/plugins/gravitywise/includes/api/form/update-field.php', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8'
            },
            body: `guid=${cwAddonGuid}&field=${field}&value=${encodeURIComponent(value)}`
        });
        return true;
    }

};