/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/

/**
 * When a new connectwise field is created in the form editor these are the default 
 * connectwise specific settings to be stored in the gravity forms field propertys.
 */
const DefaultJSON = `{
	"cwComponentField": {
		"name": "Activity",
		"endpoint": "/sales/activities"
	},
	"showAllComponents": false,
	"requiredProperties":[
	],
	"conditions": {
		"conjunction": "ANY",
		"active": false,
		"statements": [{
			"property": "",
			"operator": "=",
			"value": ""
		}]
	},
	"childConditions": {
		"conjunction": "ANY",
		"active": false,
		"statements": [{
			"property": "",
			"operator": "=",
			"value": ""
		}]
	},
	"customFieldConditions": {
		"conjunction": "ANY",
		"active": false,
		"statements": [{
			"property": "",
			"operator": "=",
			"value": ""
		}]
	},
	"orderBy": {
		"active": false,
		"statements": [{
			"property": "Id",
			"order": "ASC"
		}]
	},
	"tableSettings": {
		"active": false,
		"statements": []
	},
	"fields": {
		"active": false,
		"statements": []
	},
	"resultsViewTemplate": {
		"active": false,
		"template": ""
	},
	"allowMultiple": {
		"active": false
	},
	"limitResults": 25
}`;

const DefaultFeedJSON = `{
	"JSONType":"feed",
	"feed": {
		"Name": "",
		"ShowAllComponents": false,
		"conditions": {
			"conjunction": "ANY",
			"active": false,
			"statements": [{
				"property": "Id",
				"operator": "=",
				"value": ""
				
			}]
	}
	},
	"actions": []
}`;

var gfHelper = {
    /**
     * Gets the CW settings from the selected GF Field. It will instatiate it if it doesn't exist.
     * @returns {object} - the object containing connectwise specific settings.
     */
    getCWFieldSettings: function(){
        var gfsettings = GetSelectedField();
        if(gfsettings.cwSettings==undefined || gfsettings.cwSettings==""){
            gfsettings.cwSettings=JSON.parse(DefaultJSON);
			gfsettings.cwSettings.conditions.statements[0].property='{"name":"id","type":"integer"}';
			gfsettings.cwSettings.childConditions.statements[0].property='{"name":"id","type":"integer"}';
			gfsettings.cwSettings.customFieldConditions.statements[0].property='{"name":"id","type":"integer"}';
			gfsettings.cwSettings.tableSettings.statements[0] = {
                property: 'id',
                label: "Id"
            }; 
        }
        return gfsettings.cwSettings;
    },


	getCWFeedSettings:function(){
		var gfsettings;
		try{
			gfsettings = CWFeedAddonSettings.meta.CWFeedSettings;
		}catch(e){
			console.log(gfsettings);
			if(gfsettings==undefined || gfsettings.length==0){
				gfsettings = JSON.parse(DefaultFeedJSON);
			}
		}
		return gfsettings;
	},

	getCwFields: function(excludedId){
		var res = [];
		form.fields.forEach(element => {
			if(element.id!=excludedId || excludedId==undefined){
				res.push(element);
			}
		});
		// console.log(res);
		return res;
	},
	
	getMergetagOptions: function(excludedId,type){
		var fields = this.getCwFields(excludedId);
		
		var res = []
		var regularCW =[]
		fields.forEach(element => {
			if(!element.type.startsWith('cw_')){
				regularCW.push({property: element.label, label: element.label, tag: `{${element.label}:${element.id}}`});
				return;
			}
			if(element.cwSettings==undefined){return}
			var field = {}
			field.label = `${element.label} (id: ${element.id})`;
			field.props = [];

			element.cwSettings.properties.condition.forEach(prop => {
				if(prop.type != type && type!=undefined){
					return;
				}
				var tag = this.composeTag(element,prop.name,'');
				field.props.push({property: prop.name.replace('.',' '), tag: tag, label: element.label});	
			});
			element.cwSettings.properties.custom.forEach(prop => {
				if(prop.type!=type && type!=undefined){
					return;
				}
				var tag = this.composeTag(element,prop.name,'.customFields.*');
				field.props.push({property: 'Custom Field '+prop.name.replace('.',' '), tag: tag, label: element.label});	
			});
			
			res.push(field);
		});
		var sField = {label: 'Gravity Forms Standard Fields'};
		sField.props = regularCW;
		res.unshift(sField);
		return res;
	},
	

	composeTag: function(element, property, prefix){
		return `{${element.label}:${element.id}${prefix}.${property}}`;
	}

};