<!--************************************************************************************************-->
<!-- © Copyright 2021, BizZToolz-->
<!-- Use of this code is permitted under the terms of the GNU Lesser General Public License-->
<!-- https://www.fsf.org/-->
<!--************************************************************************************************-->

<div id="advancedSettingsVue">
    <div v-show="loading" class="loader"></div>
    <li v-if="selected!=-1">
        <!--Label Settings-->
        <label class="section_label">
            Advance CW Options Data Filters
            <?php gform_tooltip( 'adv_cw_data_filter' ) ?>
        </label>

        <!-- Conditions Section -->
        <div>
            <checkbox-label v-model="settings.conditions" :title="'Conditions'">
                <?php gform_tooltip( 'conditions_adv_filter' ) ?>
            </checkbox-label>
            <condition-field v-model="settings.conditions" :options="cwOptions.condition" :field="selected"></condition-field>
        </div>
        <!-- Child Conditions -->
        <div v-if="'child' in cwOptions && cwOptions.child.length>0">
            <checkbox-label v-model="settings.childConditions" :title="'Child Conditions'">
                <?php gform_tooltip( 'childConditions_adv_filter' ) ?>
            </checkbox-label>
            <condition-field v-model="settings.childConditions" :options="cwOptions.child" :field="selected"></condition-field>
        </div>

        <!-- Custom Field Conditions -->
        <div v-if="'custom' in cwOptions && cwOptions.custom.length>0">
            <checkbox-label v-model="settings.customFieldConditions" :title="'Custom Field Conditions'">
                <?php gform_tooltip( 'customField_adv_filter' ) ?>
            </checkbox-label>
            <condition-field v-model="settings.customFieldConditions" :options="cwOptions.custom" :field="selected"></condition-field>
        </div>

        <!-- Order By Statements -->
        <div>
            <checkbox-label v-model="settings.orderBy" :title="'Order By'">
                <?php gform_tooltip( 'orderby_adv_filter' ) ?>
            </checkbox-label>
            <orderby-field v-model="settings.orderBy" :options="cwOptions.condition"></orderby-field>
        </div>

        <!-- Fields -->
        <div>
            <checkbox-label v-model="settings.fields" :title="'Fields'">
                <?php gform_tooltip( 'fields_adv_filter' ) ?>
            </checkbox-label>
            <div style="margin:0 0 8px 2px;" v-show="settings.fields.active" v-if="'fields' in cwOptions">
                <!--See Vue Multiselect https://vue-multiselect.js.org/ -->
                <vue-multiselect v-model="settings.fields.statements" :options="cwOptions.fields" :multiple="true" :taggable="true":hide-selected="true" :option-height="30" :close-on-select="false">
                    <template slot="caret">
                    <div></div>
                    </template>
                </vue-multiselect>
            </div>
        </div>

        <!-- Results View Template -->
        <div>
            <checkbox-label v-model="settings.resultsViewTemplate" :title="'Results View Template'">
                <?php gform_tooltip( 'results_view_template' ) ?>
            </checkbox-label>
            <div style="margin:0 0 8px 2px;" v-show="settings.resultsViewTemplate.active">             
                <input class="connectwiseAdvanceComponent" style="width: 415px;"
                    type="text" placeholder="Enter results view template ..." v-model="settings.resultsViewTemplate.template"/> 
            </div>
        </div>

        <checkbox-label v-if="isTable" v-model="settings.allowMultiple" :title="'Allow Multiple Selections'">
            </checkbox-label>

        <!-- Limit Results -->
        <div>
            <div class="row">
                <label class="inline">
                    Limit
                    <?php gform_tooltip( 'limit_adv_filter' ) ?>
                </label>  
            </div>
            <div style="margin:0 0 8px 2px;">                                
                <input v-model="settings.limitResults" type="number" min="1" max="1000" class="limit">
            </div>
        </div>

        

        <!-- Table Settings -->
        <div v-if="isTable">
            <label class="section_label">
                Table Setup
                <?php gform_tooltip( 'adv_cw_data_filter' ) ?>
            </label>
            
            <div>
                <column-field v-model="settings.tableSettings" :options="cwOptions.condition"></column-field>
            
            </div>
        </div>

    </li>
</div>


<script type="text/javascript">
    const waitTime = 500;

    Vue.component('vue-multiselect', window.VueMultiselect.default);
    var advancedSettingsApp = new Vue({
        el: '#advancedSettingsVue',
        mixins: [CwMixin],
        data: {
            cwOptions: {},
            timeAtCall: 0,
            lastPromise: 0,
            isTable: false
        },
        methods:{
            getProperties: function(){
                this.loading = true;
                var timeNow = Date.now();
                if(timeNow-this.timeAtCall>waitTime){
                    cwHelperJs.retrieveConnectwiseProperties(this.settings.cwComponentField.name).then((data)=>{
                        this.cwOptions=data;
                        this.settings.properties=data;
                        this.loading=false;
                    });
                }else{
                    //Wait to stuff 
                    this.lastPromise++;
                    new Promise(resolve => {
                        var id = this.lastPromise
                        setTimeout(()=>resolve(id), waitTime)
                        

                    }).then((id)=>{
                        if(this.lastPromise==id){
                            cwHelperJs.retrieveConnectwiseProperties(this.settings.cwComponentField.name).then((data)=>{
                                this.cwOptions=data;
                                this.settings.properties=data;
                            });
                        }
                    });
                }
                this.timeAtCall=timeNow; 
            }
        },
        updated: function(){
            gform_initialize_tooltips();
        }
    });
</script>
