/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/

Vue.component('checkbox-label', {
    props: ['value','title'],
    template: 
    `<div class="row">
        <input type="checkbox" v-model="value.active"></input>
        <label class="inline">
            {{title}}
            <slot></slot>
        </label>
    </div>
    `
});

Vue.component('ibutton',{
    props: ['icon','variant'],
    computed:{
        cvariant: function(){
            if(this.variant==undefined){
                return 'button-primary'
            }
            return 'button-'+this.variant;
        }
    },
    template:`<button :class="cvariant" type="button">
    <i class="fa" :class="'fa-'+icon"></i> 
    <slot></slot>
    </button>`
});

Vue.component('flash-bold',{
    props: ['boldness','delay'],
    data: function(){
        return {
            value: true
        }
    },
    methods:{
        fontWeight: function(){
            return this.boldness!=undefined? this.boldness:600;
        }
    },
    created: function(){
        new Promise(resolve => {
            setTimeout(()=>resolve(), this.delay)
        }).then(()=>{
            this.value = false;
            this.$forceUpdate();
        });
    },
    template:`
    <div :style="{fontWeight: !value?'inherit':fontWeight()}">
    <slot >
    </slot>
    </div>
    `
});

Vue.component('action-component',{
    props: ['value','selected','order','id'],
    data: function(){
        return {
            selectedProp: "Id",
            selectedField: {},
            mtRequired: {},
            mtSelected: {}
        }
    },
    computed:{
        getFilteredFields: function(){
            return this.value.selectedFields.sort((a,b)=>{
                if(a.id<b.id){
                    return -1;
                }else{
                    return 1;
                }
            });
        }
    },
    template:`<div class="feed_container">
    <div class="feed_header" :class="{selected: selected}">
        <i class="fa fa-arrows-v" style="cursor: grab;"></i>   
        <div @click="$emit('on-select', value.id)" style="display: inline; cursor: pointer;" class="expandable"> Step {{order+1}}: Create Record in {{value.name}}</div>
        <div style="float: right; margin: auto 8px auto 0px; cursor: pointer;">
            <i class="fa fa-copy" @click="$emit('on-copy',value.id)"></i>
            <i class="fa fa-trash" @click="$emit('on-delete',value.id)"></i>
        </div>
    </div>

    <div v-show="selected">
        <div style="margin:0.5rem 0 1rem 0">
            <strong style="margin: 0">Duplicate Matching</strong>
            <select v-model="value.duplicatePolicy">
                <option  value="none">Always create a new record.</option>
                <option value="cautious">Require approval before merging duplicates.</option>
                <option value="reckless">Automatically merge duplicates.</option>
            </select>
        </div>

        <strong v-if="value.requiredFields.length!=0">Required Fields</strong>
        <div v-for="(field, j) in value.requiredFields" :key="id + field.name">
            <p class="required_field" style="margin: 0 0">{{capitalize(field.name)}}</p>
            <merge-tag @creation="configureRequired(field)" :value="mtRequired[field.name]" @input="field.field=$event.tag; setMT(mtRequired,field,$event)" ></merge-tag>

        </div>
        <div style="margin-top:1rem">
            <strong v-if="value.selectedFields.length>0">Selected Fields</strong>
        </div>
        <div v-for="(field,i) in getFilteredFields" :key="i">
            
            <p style="margin: 0 0"><flash-bold :delay="500">{{capitalize(field.name)}}</flash-bold></p>
            
            <merge-tag @creation="configureSelected(field)" :value="mtSelected[field.name]" @input="field.field=$event.tag; setMT(mtSelected,field,$event)" ></merge-tag>

            <ibutton variant="secondary" icon="close" @click.native="RemoveProperty(i)" style="margin-top:3px"></ibutton>
        </div>
        
        <div v-if="value.buildFields!==undefined" v-for="(field,i) in Object.keys(value.buildFields)" :key="field+i">
            <h3>{{field}}</h3>
            <div v-for="item in value.buildFields[field]" :key="item+field">
                <p>{{item}}</p>
                <input type="text">
            </div>
        </div>
        
        <div style="margin-top:1rem">
            <strong >Add a field</strong>
        </div>
        <div>
            <select style="width:100%" v-model="selectedProp" @keypress="AddProperty" >
                <option v-for="item in value.fields" :value="item.name">{{item.name}}</option>
            </select>
            <ibutton icon="plus" @click.native="AddProperty">Add</ibutton>
        </div>
        <div v-if="value.arrayFields!==undefined">
            <select style="width:100%" v-model="selectedField" @keypress="AddArrayField(selectedField)" >
                <option v-for="item in Object.keys(value.arrayFields)">{{item}}</option>
            </select>
            <ibutton icon="plus" @click.native="AddArrayField(selectedField)">Add Array Field</ibutton>
        </div>
    </div>
    </div>`,
    methods: {
        configureRequired: function(field){
            // console.log("SETTINGREQUIRED");
            Vue.set(this.mtRequired,field.name,{property:field.field,label:'',tag:field.field});
        },
        configureSelected: function(field){
            Vue.set(this.mtSelected,field.name,{property:field.field,label:'',tag:field.field});
        },
        setMT: function(target,field,event){
            Vue.set(target, field.name, event);
            this.$forceUpdate();
        },
        capitalize: function(string){
            
            if(string!=undefined){
                return string.charAt(0).toUpperCase() + string.slice(1);
            }
        },
        flashExpired(id){
            this.value.selectedFields.filter((el)=>{
                return el.id == id;
            })[0].flash=false;
            this.$forceUpdate();
        },
        AddProperty: function(key){
            //TODO Maintain Order
            if(this.value.fields.length==0||this.selectedProp==undefined){
                return;    
            }
            if(key.keyCode===13 || key.keyCode==undefined){
                key.preventDefault();
                var nField = this.value.fields.filter((v)=>{return v.name==this.selectedProp})[0];
                this.value.selectedFields.push(Object.assign(nField,{field: "",flash: true}));
                this.value.fields = this.value.fields.filter((v)=>{return v.name!=this.selectedProp})
                this.selectedProp = this.value.fields[0].name;
            }
        },
        RemoveProperty: function(i){
            this.value.fields.push(this.value.selectedFields[i]);
            this.value.selectedFields.splice(i,1);
        },
        AddArrayField(item){
            if(this.value.buildFields===undefined){
                this.value.buildFields={};
                Object.keys(this.value.arrayFields).forEach(el=>{
                   this.value.buildFields[el] = [];
                });
            }

            const res = {};
            Object.keys(this.value.arrayFields[item]).forEach(el=>{
                res[el] = "";
            })

            this.value.buildFields.push(res);
        }
    }
});

Vue.component('copy-modal',{
    data: function(){
        return {
            display: false,
            content: ""
        }
    },
    methods: {
        displayModal: function(content){
            this.display = true;
            this.content = content;
        }
    },
    template:
    `<div class="fb-modal" v-if="display">
        <h2>Copy</h2>
        <input type="text" onClick="this.select();" readonly v-model="content"></input>
        <button type="button" @click="display=false">Close</button>
    </div>`
});

Vue.component('paste-modal',{
    props: [],
    data: function(){
        return {
            display: false,
            content: "",
            JSONType: null
        }
    },
    methods: {
        displayModal: function(JSONType){
            this.display = true;
            this.JSONType = JSONType;
            this.content = "";
        },
        paste: function(){
            this.$emit('paste',{content: this.content, JSONType: this.JSONType})
        }
    },
    template:
    `<div class="fb-modal" v-if="display">
        <h2>Paste</h2>
        <input type="text" v-model="content"></input>
        <div>
            <button type="button" @click="paste">OK</button>
            <button type="button" @click="display=false">Close</button>
        </div>
    </div>`
});

// .fb-modal {
// 	position: fixed;
// 	width: 20rem;
// 	height: 7rem;
// 	background-color: white;
// 	box-shadow: 0rem 0rem 0rem 1000rem rgba(0,0,0,0.6);
// 	left: calc(50% - 10rem);
// 	top: calc(50% - 3.5rem);
// 	display: flex;
// 	flex-direction: column;
// 	padding: 1rem;
// }








