/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/

class cwField{

    static AppList = [];
    constructor(fieldId, settings, queryAutomatically = false, selectDefault = false){
        var dependencies = cwField.GetDependencies(settings.advancedFilters);
        this.vueApp = new Vue({
            el: '#vField_' + fieldId,
            data: {
                fieldId: fieldId,
                isLive: false,
                isCreated: false,
                searchString: '',
                value: '',
                query: [],
                loading: true,
                selected: {},
                chosen: false,
                cwSettings: settings,
                QueryAutomatically: queryAutomatically,
                selectDefault: selectDefault,
                dependencies: dependencies
            },
            computed: {
                firstOrEmpty: function(){
                    if(this.query != undefined && this.query.length != 0){
                        return this.calcResView(this.query[0]) ;
                    }else{
                        return 'N/A';
                    }
                }
            },
            methods: {
                search: async function(initial=false){
                    this.loading = true;
                    try {
                        var data = await cwHelperJs.retrieveDatabaseLookup(fieldId,this.searchString)
                        this.query = data;
                        this.loading = false;
                        if(this.selectDefault){
                            if(this.query!=undefined && this.query.length!=0){
                                await this.Click(this.query[0],false,!initial);
                            }
                        }
                    } catch (e) {
                        this.query = [];
                        this.loading = false;
                    }
                },
                //override default enter behavior.
                searchKey: function(e){
                    if(e.keyCode==13){
                        e.preventDefault();
                        this.search();
                    }
                },
                Click: async function(selected, useJson=false, updatec=true){
                    if(useJson && selected != ""){
                            selected = JSON.parse(selected);
                    }
                   
                    this.selected = selected;
                    this.value = selected;
                    this.chosen=true;
                    await this.FieldChange();
                    if(updatec){
                        cwField.UpdateChildren(this.fieldId);
                    }
                },
                FieldChange: async function(){
                    var res;
                    try {
                        res = JSON.stringify(this.value);
                    } catch (e) {
                        res = "";
                    }
                    await cwHelperJs.notifyMergeTagChange(fieldId,res);
                },
                calcResView: function(item){
                    if(this.cwSettings.resultsViewTemplate.active){
                        return MergeTagsJS.ConvertResultsView(this.cwSettings.resultsViewTemplate.template,item);
                    }else if(this.cwSettings.fields.active){
                        var res = '';
                        this.cwSettings.fields.statements.forEach(element => {
                            if(item[element]==undefined){return;}
                            if(typeof item[element] !== 'object'){
                                res+=item[element] + ' ';
                            }else{
                                res+=item[element]['name'] + ' ';
                            }
                        });
                        return res;                    
                    }else{
                        return `${item[Object.keys(item)[0]]}, ${item[Object.keys(item)[1]]}`;
                    }
                },
                explodeCWObject: function(item){
                    if(typeof item !== 'object'){
                        return item
                    }else{
                        return item['name']
                    }
                },
                triggerUpdate: function(){
                    cwField.UpdateChildren(this.fieldId);
                },
                AddOrRemoveValue: async function(value,add){
                    if(this.value === ''){
                        this.value = [];
                    }
                    
                    if(add){
                        this.value.push(value);
                    }else{
                        var i = this.value.indexOf(value);
                        if(i>-1){
                            this.value.splice(i,1);
                        }
                    }

                    await this.FieldChange();
                    cwField.UpdateChildren(this.fieldId);
                },
                log: function(value){
                    console.log(value);
                },
                calculateSelection: async function(selectedOptions){
                    this.value = [];
                    for (let index = 0; index < selectedOptions.length; index++) {
                        const element = selectedOptions[index];
                        this.value.push(element.value);
                        
                    }
                    await this.FieldChange();
                    cwField.UpdateChildren(this.fieldId);
                    
                },
                ClickAdvanced: async function(item,allowMultiple, ctrlDown){
                    if(!allowMultiple){
                        this.Click(item);
                        return;
                    }
                    

                    if(!ctrlDown || this.value == ""){
                        this.value = [];
                    }
                    var i = this.value.indexOf(item);
                    if(i>-1){
                        this.value.splice(i,1);
                    }else{
                        this.value.push(item);
                    }

                    await this.FieldChange();
                    cwField.UpdateChildren(this.fieldId);
                },
                tryHasValue: function(item){
                    if(this.cwSettings.allowMultiple.active && typeof this.value =="object"){
                        return this.value.indexOf(item)!=-1;
                    }else{
                        return this.value==item;
                    }
                }
            },
            created: function(){
                if(typeof cwAddonGuid !== 'undefined'){
                    this.isLive = true;
                }
            }        
        });
        cwField.AppList.push(this.vueApp);
    }

    static async UpdateChildren(fieldId){
        // console.log("Updating Children of Field "+ fieldId);
        for (let i = 0; i < cwField.AppList.length; i++) {
            const element = cwField.AppList[i];
            
            if (element.QueryAutomatically) {
                if (element.dependencies.includes(fieldId)) {
                    // console.log(`Update ${fieldId} triggered on Field ${element.fieldId}`);
                    element.search();
                }   
            }
        }
    }

    static GetDependencies(filter){
        var res = [];
        var tags = (filter ?? '').matchAll(/%7B.*?%7D/g);
        for(const element of tags) {
            var t = decodeURIComponent(element[0]).match(/(?<=:)\d+/);
            res.push(parseInt(t[0]));
        }
        return res;
    }
}
