/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/

Vue.component('multi-statement-field', {
    props: ['value', 'display', 'spacing'],
    template: 
    `<div style="margin-bottom: 16px;">
        <div v-for="(item,index) in value.statements" :style="{display: display, marginTop: spacing, marginBottom: spacing}">
            <slot v-model="value.statements[index]" v-bind:child="item">
            </slot>
            
            <a @click="ChangeCount(value.statements,index)" style="margin:auto">
                <i :class="{'gficon-subtract': !isLastElement(value.statements,index) , 'gficon-add': isLastElement(value.statements,index)}"></i>
            </a>
        </div>
    </div>`,
    methods: {
        ChangeCount: function (arr,i){
            if(this.isLastElement(arr,i)){
                this.$emit('add-statement',arr);
            }else{
                arr.splice(i,1);
            }
        },
        isLastElement(arr, index){
            return index == arr.length-1
        }
    }
});




Vue.component('condition-field', {
    props: ['value','options','field'],
    data: function(){
        return {
            operators: ['=','!=','<','>']

        };
    },
    template: 
    `<div v-show="value.active">
        <div v-show="value.statements.length>1">
            Show if <select v-model="value.conjunction">
                <option value="ALL">All</option>
                <option value="ANY">Any</option>
            </select> values match.
        </div>

        <multi-statement-field v-model="value" v-slot="child" @add-statement="pushDefault($event)" :display="'block'" :spacing="'8px'">
            <select class="cwFilterFields" style="width: 341px;" v-model="child.child.property">
    
                <option v-for="opt in options" :value="JSON.stringify(opt)">{{opt.name}}</option>
                
            </select>
            <select class="cwOperators" style="width: 70px;"  v-model="child.child.operator">

                <option v-for="op in operators" :value="op">{{op}}</option>

            </select>
            
            <merge-tag :value="child.child.value" @input="child.child.value=$event" :field="field"></merge-tag>
            
        </multi-statement-field>
        
    </div>
    `,
    methods: {
        pushDefault: function(arr){
            arr.push({
                property: '{"name":"id","type":"integer"}',
                operator: '=',
                value: ''
            });
        }
    }
});

Vue.component('dual-condition-field', {
    props: ['value','options'],
    data: function(){
        return {
            operators: ['=','!=','<','>']

        };
    },
    template: 
    `<div v-show="value.active">
        <div v-show="value.statements.length>1">
            Show if <select v-model="value.conjunction">
                <option value="ALL">All</option>
                <option value="ANY">Any</option>
            </select> values match.
        </div>

        <multi-statement-field v-model="value" v-slot="child" @add-statement="pushDefault($event)">
            <merge-tag :value="child.child.property" @input="setProperty(child,$event)"></merge-tag>

            <select class="cwOperators" style="width: 70px;"  v-model="child.child.operator">

                <option v-for="op in operators" :value="op">{{op}}</option>

            </select>
            
            <merge-tag :value="child.child.value" @input="setChild(child,$event)"></merge-tag>
            
        </multi-statement-field>
        
    </div>
    `,
    methods: {
        setChild: function(child, val){
            Vue.set(child.child,'value',val);
        },
        setProperty: function(child,val){
            Vue.set(child.child,'property',val);
        },
        pushDefault: function(arr){
            arr.push({
                property: '{"name":"id","type":"integer"}',
                operator: '=',
                value: ''
            });
        }
    }
});



Vue.component('orderby-field', {
    props: ['value','options'],
    template: 
    `<div v-show="value.active">
        <multi-statement-field v-model="value" v-slot="child" @add-statement="pushDefault($event)" :display="'block'" :spacing="'8px'">
            <select class="cwFilterFields" style="width:342px;" v-model="child.child.property">
                <option v-for="cwOption in options" :value="cwOption.name">{{cwOption.name}}</option>
            </select>
            <select v-model="child.child.order" style="width:70px;">
                <option value="ASC">A→Z</option>
                <option value="DESC">Z→A</option>
            </select>
        </multi-statement-field>
    </div>
    `,
    methods: {
        pushDefault: function(arr){
            arr.push({
                property: 'Id',
                order: "ASC"
            });
        },
    }
});

Vue.component('column-field', {
    props: ['value','options'],
    template: 
    `<div v-show="value.active">
        <div style="display:flex;">
            <p style="width:250px;text-align: center;margin: .1em;">Column Property</p>
            <p style="width:165px;text-align: center;margin: .1em;">Column Label</p>
        </div>
        <multi-statement-field v-model="value" v-slot="child" @add-statement="pushDefault($event)" :display="'flex'">
            <select class="cwFilterFields" style="width:250px;" v-model="child.child.property">
            <option v-for="cwOption in options" :value="cwOption.name">{{cwOption.name}}</option>
            </select>
            <input type="text" v-model="child.child.label" />
        </multi-statement-field>
    </div>
    `,
    methods: {
        pushDefault: function(arr){
            arr.push({
                property: 'id',
                label: "Id"
            });
        },
    }
});