<!--************************************************************************************************-->
<!-- © Copyright 2021, BizZToolz-->
<!-- Use of this code is permitted under the terms of the GNU Lesser General Public License-->
<!-- https://www.fsf.org/-->
<!--************************************************************************************************-->


<div id="VueLoader"  class="loader"></div>
<div id="VueFeedBuilder">
    
    <copy-modal ref="copyModal"></copy-modal>
    <paste-modal ref="pasteModal" @paste="processPastedJSON($event.JSONType,JSON.parse($event.content))"></paste-modal>
    <div class="feed_container">
        <p class="feed_header selected">Feed Settings
        <p>
        <table class="feedGrid">
            <tbody>
                <tr>
                    <td class="required_field">Name</td>
                    <td><Input type="text" name="_gform_setting_name" v-model="feedSettings.feed.Name"></Input></td>
                </tr>
                <tr>
                    <td>Enable Conditional Logic</td>
                    <td><input type="checkbox" v-model="feedSettings.feed.conditions.active"></input></td>
                </tr>
                <tr v-show="feedSettings.feed.conditions.active">
                    <td>Conditions</td>
                    <td>
                        <dual-condition-field v-model="feedSettings.feed.conditions" :options="testOptions"></dual-condition-field>
                    </td>
                </tr>
                <tr>
                </tr>
                <tr>
                    <td></td>
                    <td>
                        <ibutton @click.native="copyJSON('feed')" icon="copy">Copy Feed</ibutton>
                        <ibutton icon="paste" @click.native="pasteJSON('feed')">Paste Feed</ibutton>
                        <ibutton icon="bug" @click.native="debug=!debug">Debug</ibutton>
                        <ibutton icon="link"><a style="color: white;" :href="getLink">Retries</a></ibutton>
                    </td>
                </tr>
            </tbody>
        </table>
    </div>
    <div class="feed_container_empty" v-if="feedSettings.actions.length>0">
        <p class="feed_header_break top">Start Feed</p>
    </div>
    <div class="feed_container_empty" v-if="feedSettings.actions.length==0">
        <p class="feed_header_break top bottom">No actions added - Feed will not be processed</p>
    </div>
    <draggable v-model="feedSettings.actions" animation="100" @start="drag = true" @end="drag = false" handle=".fa-arrows-v">
        <transition-group type="transition" :name="!drag ? 'sortable' : null">
            <action-component v-for="(el,i) in feedSettings.actions" v-model="feedSettings.actions[i]" :selected="el.id==selected" :order="i" :key="el.id" :id="el.id" @on-copy="copyJson('action',$event)" @on-select="selectAction($event)" @on-delete="removeAction($event)">
            </action-component>

        </transition-group>
    </draggable>
    <div class="feed_container_empty" v-if="feedSettings.actions.length>0">
        <p class="feed_header_break bottom">End Feed</p>
    </div>

    <div class="feed_container">
        <p class="feed_header selected">Add a New Action
        <p>
        <p>Show all?
            <input type="checkbox" v-model="feedSettings.feed.ShowAllComponents" @click="getConnectwiseComponents(!feedSettings.feed.ShowAllComponents)"></input>
        <p>
            <select v-model="selectedComponent">
                <option v-for="item in components" :value="item">
                    {{item.name}}
                </option>
            </select>
            <button class="button-primary" type="button" @click="addAction"><i class="fa fa-plus"></i> Add</button>
            <button class="button-primary" type="button" @click="pasteJSON('action')"><i class="fa fa-clipboard"></i> From Clipboard</button>
    </div>
    <div class="feed_container" v-if="debug">
        <p class="feed_header">Vue JSON
        <p>
        <pre>
        {{JSON.stringify(feedSettings,null,2)}}
        </pre>
    </div>
    <input name="_gform_setting_CWFeedSettings" :value="getFeedJSON" hidden></input>
</div>



<script type="text/javascript">
    const FeedAsActionsWarning = 'You are trying to paste a form as an action, are you sure you want to do this?';

    Vue.component('vue-multiselect', window.VueMultiselect.default);
    var VueFeedBuilder = new Vue({
        el: '#VueFeedBuilder',
        data: {
            components: [],
            selectedComponent: '/sales/activities',
            selectedProperties: [],
            testOptions: ['Name', 'ID', 'Author'],
            drag: false,
            debug: false,
            selected: -1,
            testValue: {
                active: true,
                statements: [{
                    property: 'Id',
                    operator: '=',
                    value: ''
                }]
            },
            feedSettings: {},
        },
        computed: {
            getFeedJSON: function() {
                return JSON.stringify(this.feedSettings);
            },
            getLink: function(){
                return '/wp-admin/admin.php?page=feed_retrials&feed_id='+feed_id;
            }
        },
        methods: {
            //Helper functions
            //TODO move functions out of scope, let's be functional.

            getNextID: function(arr) {
                var lastId = 0;
                arr.forEach(element => {
                    if (lastId < element.id) {
                        lastId = element.id;
                    }
                });
                return lastId + 1;
            },
            getComponentFromEndpoint: function(arr, endpoint) {
                return arr.filter(item => {
                    return item.endpoint == endpoint;
                })[0];
            },
            getActionByID: function(arr, id) {
                return arr.filter(item => {
                    return item.id == id;
                })[0];
            },
            getDefaultAction: function(id, component) {
                return {
                    JSONType: 'action',
                    id: id,
                    endpoint: component.endpoint,
                    name: component.name,
                    fields: [],
                    requiredFields: [],
                    selectedFields: [],
                    duplicatePolicy: 'none'
                };

            },
            getPermissions: function(permission, callback) {

                navigator.permissions.query({
                    name: "clipboard-write"
                }).then(result => {
                    if (result.state == "granted" || result.state == "prompt") {
                        callback(true);
                    }
                }, reject => {
                    callback(false);
                });

            },

            //Actual Methods
            addAction: function() {
                var nextId = this.getNextID(this.feedSettings.actions);


                var newAction = this.getDefaultAction(nextId, this.selectedComponent);
                this.feedSettings.actions.push(newAction);


                cwHelperJs.retrieveRequiredFields(this.selectedComponent.name).then(data => {
                    data.required.forEach(element => {
                        newAction.requiredFields.push({
                            name: element,
                            field: ""
                        });
                    });
                    newAction.arrayFields = data.arrayFields;
                    var i = 0;
                    Object.keys(data.properties).forEach(key => {
                        //exclude if its already in required
                        var isRequired = false;
                        newAction.requiredFields.forEach(element => {
                            if (element.name == key) {
                                isRequired = true;
                            }
                        });
                        if (isRequired) {
                            return
                        }
                        //exclude of its type is an array
                        if (data.properties[key].type == "object" || data.properties[key].type == "array") {
                            return;
                        }
                        var res = Object.assign({
                            name: key,
                            id: i
                        }, data.properties[key]);
                        newAction.fields.push(res);
                        i++;
                    });

                });

                this.selected = nextId;
            },
            selectAction: function(id) {
                if (this.selected == id)
                    this.selected = -1;
                else
                    this.selected = id;
            },
            removeAction: function(id) {
                var actionToRemove = this.getActionByID(this.feedSettings.actions, id);
                var actionIndex = this.feedSettings.actions.indexOf(actionToRemove);
                this.feedSettings.actions.splice(actionIndex, 1);
            },
            getConnectwiseComponents: function(showAll) {
                cwHelperJs.retrieveConnectwiseComponents(showAll)
                    .then((data) => {
                        this.components = data;
                    });
            },
            copyJSON: function(JSONType, id) {
                this.getPermissions('clipboard-write', (perm) => {
                    switch (JSONType) {
                        case 'feed':
                            if (perm) {
                                navigator.clipboard.writeText(JSON.stringify(this.feedSettings, null, '\t'));
                            } else {
                                this.$refs.copyModal.displayModal(JSON.stringify(this.feedSettings, null, '\t'));
                            }
                            break;
                        case 'action':
                            var action = this.getActionByID(this.feedSettings.actions, id);
                            if (perm) {
                                navigator.clipboard.writeText(JSON.stringify(action, null, '\t'));
                            } else {
                                this.$refs.copyModal.displayModal(JSON.stringify(action, null, '\t'));
                            }
                            break;
                    }
                });
            },
            pasteJSON: function(JSONType) {
                this.getPermissions('clipboard-read', (perm) => {
                    if (perm) {
                        navigator.clipboard.readText().then(text => {
                            try {
                                this.processPastedJSON(JSONType, JSON.parse(text));
                            } catch (e) {
                                alert('No feed in clipboard.' + e);
                            }
                        });
                    } else {
                        this.$refs.pasteModal.displayModal(JSONType)
                    }
                });
            },
            processPastedJSON: function(JSONType, pastedObj) {
                // console.log("PASTE");

                var nextId = this.getNextID(this.feedSettings.actions);
                switch (JSONType) {
                    case 'feed':
                        if (pastedObj.JSONType == JSONType)
                            this.feedSettings = pastedObj;
                        break;
                    case 'action':
                        if (pastedObj.JSONType == JSONType) {
                            pastedObj.id = nextId;
                            this.feedSettings.actions.push(pastedObj);
                        } else if (pastedObj.JSONType == 'feed') {
                            if (confirm(FeedAsActionsWarning) == false) return;

                            pastedObj.actions.forEach(element => {
                                element.id += nextId;
                                this.feedSettings.actions.push(element);
                            });
                        }
                        break;
                }
            }

        }
    });

    var feed_id = 0;
    jQuery(document).ready(async function() {
        VueFeedBuilder.getConnectwiseComponents(false);

        const queryString = window.location.search;
        const urlParams = new URLSearchParams(queryString);
        if (urlParams.has('fid')) {
            feed_id = urlParams.get('fid');

            var response = await fetch(`/wp-content/plugins/gravitywise/includes/api/feed-builder/get-settings.php?id=` + feed_id);
            var m = await response.json();
            var r = m;
            VueFeedBuilder.feedSettings = r;
            VueFeedBuilder.$forceUpdate();
            jQuery('#VueLoader').remove();
        }

    });
</script>