<!--************************************************************************************************-->
<!-- © Copyright 2021, BizZToolz-->
<!-- Use of this code is permitted under the terms of the GNU Lesser General Public License-->
<!-- https://www.fsf.org/-->
<!--************************************************************************************************-->

<div id="feedBuilderVue">
	<div v-if="!Loaded" class="loader"></div>
	<div v-if="Loaded">
		<feed-settings></feed-settings>
        <div class="fb_flow_container">
            <div class="fb_flow_shape disable-select">
                START
            </div>
            <div class="fb_flow_connector"></div>
            <template v-for="(item,i) in FeedSettings.actions">
                <div class="fb_flow_shape fb_flow_shape--large" :style="calcWidth(item.component.name)">
                    <a @click="loadingViewAction=i;$refs.modal.showModal({...item, index: i})" class="disable-select">{{item.type}} {{item.component.name}}</a>
                    <div v-if="loadingViewAction==i" class="loader"></div>
                    <a v-if="loadingViewAction!=i" class="fb_flow_control" @click="moveUp(i)"><i class="fa fa-chevron-up"></i></a>
                    <a v-if="loadingViewAction!=i" class="fb_flow_control" @click="moveDown(i)"><i class="fa fa-chevron-down"></i></a>
                    <a v-if="loadingViewAction!=i" class="fb_flow_control" @click="deleteAction(i)"><i class="fa fa-trash" style="margin-left: 8px;"></i></a>
                </div>
                <div class="fb_flow_connector"></div>
            </template>
            <div v-if="loadingAddAction" class="loader"></div>
            <div v-if="loadingAddAction" class="fb_flow_connector"></div>
            <div class="fb_flow_shape disable-select">
                END
            </div>
        </div>
        <action-modal @loaded="loadingViewAction=-1;" ref="modal"></action-modal>
        <add-action @adding="loadingAddAction=true;" @added="loadingAddAction=false;"></add-action>
	</div>

    <input name="_gform_setting_CWFeedSettings" :value="JSON.stringify(FeedSettings)" type="text" hidden></input>
</div>


<script type="module">
	import store from '/../wp-content/plugins/gravitywise/public/js/vue/feed-builder/store/store.js';
	import '/../wp-content/plugins/gravitywise/public/js/vue/feed-builder/organisms/feed-settings.js';
    import '/../wp-content/plugins/gravitywise/public/js/vue/field-settings/atoms/merge-tag.js';
    import '/../wp-content/plugins/gravitywise/public/js/vue/feed-builder/organisms/add-action.js';
    import '/../wp-content/plugins/gravitywise/public/js/vue/feed-builder/organisms/action-modal.js';
    import '/../wp-content/plugins/gravitywise/public/js/vue/feed-builder/molecules/request-display.js';
    import '/../wp-content/plugins/gravitywise/public/js/vue/feed-builder/atoms/object-display.js';
    import '/../wp-content/plugins/gravitywise/public/js/vue/feed-builder/atoms/array-display.js';
    import '/../wp-content/plugins/gravitywise/public/js/vue/feed-builder/atoms/field-entry.js';
    import '/../wp-content/plugins/gravitywise/public/js/vue/feed-builder/atoms/copy-modal.js';
    import '/../wp-content/plugins/gravitywise/public/js/vue/feed-builder/atoms/paste-modal.js';
    import '/../wp-content/plugins/gravitywise/public/js/vue/feed-builder/molecules/dual-condition.js';
    import '/../wp-content/plugins/gravitywise/public/js/vue/field-settings/atoms/multi-statement-field.js';
    import '/../wp-content/plugins/gravitywise/public/js/vue/feed-builder/atoms/item-lookup.js';


    const fieldSettingsApp = new Vue({
        el: '#feedBuilderVue',
        store,
        data: {
            loadingViewAction: -1,
            loadingAddAction: false
        },
	    computed: {
            ...Vuex.mapState(['Loaded','FeedSettings','selectedActionIndex'])
	    },
        methods: {
            calcWidth(name) {
                let width = 230 + ((name.length - 7) * 9.33);
                return {
                    width: width + 'px'
                }
            },
            moveUp(i) {
                if (i > 0) {
                    this.$store.commit('moveActionUp',i);
                    this.$forceUpdate();
                }
            },
            moveDown(i) {
                if (i < this.FeedSettings.actions.length - 1) {
                    this.$store.commit('moveActionDown',i);
                    this.$forceUpdate();
                }
            },
            deleteAction() {
                if (confirm("This will delete the current Action. Are you sure?")) {
                    this.$store.commit('deleteAction',this.selectedActionIndex);
                }
            }
        },
	    created: function () {
                const queryString = window.location.search;
                const urlParams = new URLSearchParams(queryString);
                this.$store.dispatch('fetchSettings',urlParams.get('fid'));
                this.$store.dispatch('fetchComponents',false);
	    }
    });

    jQuery('#gform-settings-save').hover(()=>{
        fieldSettingsApp.$forceUpdate();
    }).click(()=>{
        isDirty = false;
    });

    jQuery(document).on('keypress', '.gform-settings__content', function (e) {
        if (e.keyCode == 13) {
            e.preventDefault();
            return false;
        }
    });

    let isDirty = false;
    const vueDiv = document.getElementById('feedBuilderVue');
    vueDiv.addEventListener('change', () => {
        isDirty = true;
    })

    window.addEventListener("beforeunload", (e) => {
        if (!isDirty) {
            return;
        }
        e.returnValue = true;
        return true;
    });

</script>