/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/

Vue.component('action-modal', {
    data: function (){
        return{
            displayed: false,
            sProp: '',
        }
    },
    computed: {
        ...Vuex.mapState(['FeedSettings','Components','Properties','selectedActionIndex']),
        ...Vuex.mapGetters(['SelectedAction'])
    },
    methods:{
        showModal: async function (id){
            await this.$store.dispatch('changeSelectedAction',id);
            document.body.style.overflow = 'hidden';
            this.displayed=true;
            this.$forceUpdate();
            this.$emit('loaded');
        },
        trimLeaves: function (object){
            var copy = {...object};
            Object.keys(copy).forEach(el=>{
                if(typeof copy[el] !=="object"){
                    copy[el] = '';
                }else{
                    copy[el] = this.trimLeaves(copy[el]);
                }
            })
            return copy;
        },
        addField() {
            if (this.sProp) {
                var prop = this.Properties[this.SelectedAction.component.name][this.sProp];
                if(Array.isArray(prop)){
                    prop = [this.trimLeaves(prop[0])];
                }else if(typeof prop === "object"){
                    prop = this.trimLeaves(prop);
                }else{
                    prop = '';
                }

                if (!this.SelectedAction.request[this.sProp]) {
                    this.SelectedAction.request[this.sProp] = prop;
                }
                this.$forceUpdate();
                this.$refs.reqDisp.$forceUpdate();
            } else {
                alert('Select a Field before adding');
            }
        },
        getPermissions: function(permission, callback) {

            navigator.permissions.query({
                name: "clipboard-write"
            }).then(result => {
                if (result.state == "granted" || result.state == "prompt") {
                    callback(true);
                }
            }, reject => {
                callback(false);
            });

        },
        copyJSON: function() {
            this.getPermissions('clipboard-write', (perm) => {
                    if (perm) {
                        navigator.clipboard.writeText(JSON.stringify(this.SelectedAction, null, '\t'));
                    } else {
                        this.$refs.copyModal.displayModal(JSON.stringify(this.SelectedAction, null, '\t'));
                    }
            });
        },
        pasteJSON: function() {
            this.getPermissions('clipboard-read', (perm) => {
                if (perm) {
                    navigator.clipboard.readText().then(text => {
                        try {
                            this.processPastedJSON(JSON.parse(text));
                        } catch (e) {
                            alert('No feed in clipboard.' + e);
                        }
                    });
                } else {
                    this.$refs.pasteModal.displayModal()
                }
            });
        },
        processPastedJSON: function(pastedObj) {
            if(pastedObj['component'] !== null){
                this.SelectedAction['component'] = pastedObj['component'];
                this.SelectedAction['request'] = pastedObj['request'];
                this.SelectedAction['duplicatePolicy'] = pastedObj['duplicatePolicy'] ?? "none";
            }
            this.$store.dispatch('updateMT');
        },
        deleteAction() {
            if (confirm("This will delete the current Action. Are you sure?")) {
                this.displayed=false;
                this.$store.commit('deleteAction',this.selectedActionIndex);
            }
        },
        closeModal() {
            this.displayed=false;
            this.$parent.$forceUpdate();
            document.body.style.overflow = 'auto';
        }
    },
    template: `
        <div class="fb_action_modal" v-if="displayed">
            <copy-modal ref="copyModal"></copy-modal>
            <paste-modal ref="pasteModal" @paste="processPastedJSON(JSON.parse($event.content))"></paste-modal>
            <div class="fb_modal_header">
                <h1>
                    {{SelectedAction.type}} {{SelectedAction.component.name}}
                </h1>
                <h1>
                    <a @click="copyJSON" style="cursor: pointer">
                        <i class="fa fa-copy"></i>
                    </a>
                    <a @click="pasteJSON" style="cursor: pointer">
                        <i class="fa fa-paste"></i>
                    </a>
                    <a @click="deleteAction();" style="cursor: pointer">
                        <i class="fa fa-trash"></i>
                    </a>
                    <a @click="closeModal();" style="cursor: pointer">
                        <i class="fa fa-close"></i>
                    </a>
                </h1>
            </div>
            <div class="fb_modal_body">
                
                <div style="margin:0.5rem 0 1rem 0">
                    <strong style="margin: 0">Duplicate Matching</strong>
                    <select v-model="SelectedAction.duplicatePolicy">
                        <option value="none">Always create a new record.</option>
                        <option value="cautious">Require approval before merging duplicates.</option>
                        <option value="reckless">Automatically merge duplicates.</option>
                    </select>
                </div>
            
                <ul class="fb_settings_list">
                    <li>
                        <p>Fields</p>
                        <select v-model="sProp">
                            <option v-for="item in Object.keys(Properties[SelectedAction.component.name])">{{item}}</option>
                        </select>    
                        <button class="button gform-button gform-button--white" type="button" @click="addField">Add</button>
                    </li>
                    <br/>
                    <request-display ref="reqDisp">
                    </request-display>
                </ul>
            </div>
        </div>
    `
});