/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/

Vue.component('feed-settings', {
    computed: {
        ...Vuex.mapState(['FeedSettings'])
    },
    created: function (){
        if(typeof this.FeedSettings.email === 'undefined'){
            this.FeedSettings.email = {
                active: false,
                onSubmit: false,
                onFail: false,
                onSuccess: false,
                addresses: ''
            };
        }
    },
    methods: {
        getPermissions: function(permission, callback) {

            navigator.permissions.query({
                name: "clipboard-write"
            }).then(result => {
                if (result.state == "granted" || result.state == "prompt") {
                    callback(true);
                }
            }, reject => {
                callback(false);
            });

        },
        copyJSON: function() {
            this.getPermissions('clipboard-write', (perm) => {
                if (perm) {
                    navigator.clipboard.writeText(JSON.stringify(this.FeedSettings, null, '\t'));
                } else {
                    this.$refs.copyModal.displayModal(JSON.stringify(this.FeedSettings, null, '\t'));
                }
            });
        },
        pasteJSON: function() {
            this.getPermissions('clipboard-read', (perm) => {
                if (perm) {
                    navigator.clipboard.readText().then(text => {
                        try {
                            this.processPastedJSON(JSON.parse(text));
                        } catch (e) {
                            alert('No feed in clipboard.' + e);
                        }
                    });
                } else {
                    this.$refs.pasteModal.displayModal()
                }
            });
        },
        processPastedJSON: function(pastedObj) {
            if(pastedObj['name'] !== null){
                this.$store.commit('setFeedSettings',pastedObj);
            }
            this.$store.dispatch('updateMT');
        }
    },
    template: `
        <div>
            <copy-modal ref="copyModal"></copy-modal>
            <paste-modal ref="pasteModal" @paste="processPastedJSON(JSON.parse($event.content))"></paste-modal>
            <ul class="fb_settings_list">
                <li>
                    <p>Feed Name</p>
                    <input  name="_gform_setting_name" v-model="FeedSettings.name" type="text">    
                </li>
                <li>
                    <p>Enable Conditional Logic</p>
                    <input v-model="FeedSettings.conditions.active" type="checkbox">               
                </li>
                <li>
                    <dual-condition-field-u v-model="FeedSettings.conditions">
                    </dual-condition-field-u>
                </li>
                <li>
                    <p>Enable Email Notifications</p>
                    <input v-model="FeedSettings.email.active" @change="$forceUpdate()" type="checkbox">
                </li>
                <li>
                    <button class="primary button fb_margin" type="button" @click="copyJSON">
                        Copy <i class="fa fa-copy"></i>
                    </button>
                    <button type="button" class="primary button fb_margin" @click="pasteJSON">
                        Paste <i class="fa fa-paste"></i>
                    </button>
                </li>
            </ul>
        
        </div>
    `
});