/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/

const apiPath = '/../wp-content/plugins/gravitywise/includes/api';

const defaultAction = {
    id: 1,
    type: 'Create',
    component: {
        name: 'Blarg',
        endpoint: 'Blarg'
    },
    request: {

    },
    duplicatePolicy: 'none'
}

const store = new Vuex.Store({
    state: {
        Loaded: false,
        FeedSettings: {
            name: 'Untitled Feed',
            conditions: {
                active: false,
                conjunction: 'ANY',
                statements: [
                    {
                        property: '',
                        operator: '=',
                        value: ''
                    }
                ]
            },
            email: {
              active: false,
              onSubmit: false,
              onFail: false,
              onSuccess: false,
              addresses: ''
            },
            actions: [

            ]
        },
        Properties: {

        },
        RequiredProps:{

        },
        Components: [

        ],
        MergeTags: {

        },
        selectedActionIndex: 0


    },
    mutations: {
        addProperties(state, {component,properties}){
            state.Properties[component] = properties;
        },
        setComponents(state, components){
            state.Components = components;
        },
        setFeedSettings(state, settings ){
            state.FeedSettings = settings;
            state.Loaded = true;
        },
        setLoaded(state, isLoaded) {
            state.Loaded = isLoaded;
        },
        changeMT(state, value) {
            var lookbackTags = {};
            var i=0;
            state.FeedSettings.actions.forEach(el =>{
                i++;
                var props = {};

                Object.keys(state.Properties[el.component.name]).forEach(property=>{
                    props[property] = `{Previously Created ${el.component.name}:<${i}.${property}}`;
                });
                lookbackTags[`Step ${i} - ${el.component.name}`] = props;
            })

            state.MergeTags = {...lookbackTags, ...value};
        },
        addAction(state, {component,type,required}){
            const newAction = JSON.parse(JSON.stringify(defaultAction));
            newAction.id = 1;
            newAction.type = type;
            newAction.component = component;
            required.forEach(el=>{
               newAction.request[el] = '';
            });
            state.RequiredProps = required;
            state.FeedSettings.actions.push(newAction);
        },
        changeSelectedAction(state, {index,required}){
            required.forEach(el=>{
                if(!el in state.FeedSettings.actions[index].request){
                    state.FeedSettings.actions[index].request[el] = '';
                }
            });
            state.RequiredProps = required;
            state.selectedActionIndex = index;
        },
        moveActionUp(state, index) {
            [state.FeedSettings.actions[index-1], state.FeedSettings.actions[index]] = [state.FeedSettings.actions[index], state.FeedSettings.actions[index-1]];
        },
        moveActionDown(state, index) {
            [state.FeedSettings.actions[index], state.FeedSettings.actions[index+1]] = [state.FeedSettings.actions[index+1], state.FeedSettings.actions[index]];
        },
        deleteAction(state, index) {
            state.FeedSettings.actions.splice(index, 1);
        }
    },
    getters: {
        SelectedAction(state){

            return state.FeedSettings.actions[state.selectedActionIndex];
        }
    },
    actions: {
        async changeAction(context, id){

            context.commit('changeSelectedAction',id);
        },
        async fetchComponents(context, all){
            const request = await fetch(`${apiPath}/common/get-components.php?all=${all}`);
            context.commit('setComponents',await request.json());
        },
        async fetchProperties(context, component){
            const request = await fetch(`${apiPath}/common/get-builder-properties.php?component=${component}`);
            context.commit('addProperties',{component, properties: await request.json()});
        },
        async fetchSettings(context, fid){
            const response = await fetch(`${apiPath}/feed-builder/get-settings.php?id=${fid}`);
            const resJson = await response.json();
            var total = resJson.actions.length;
            if (total == 0) {
                context.commit('setFeedSettings',resJson);
                context.dispatch('updateMT');
            }
            resJson.actions.forEach(el=>{
                context.dispatch('fetchProperties',el.component.name).then((value)=>{
                   total--;
                   if(total==0){
                       context.commit('setFeedSettings',resJson);
                       context.dispatch('updateMT');
                   }
                });
            });

        },
        async updateMT({commit, state, dispatch}){
            for (let i = 0; i < state.FeedSettings.actions.length; i++) {

                var comp = state.FeedSettings.actions[i];
                if(state.Properties[comp.component.name] === undefined) {
                    await dispatch('fetchProperties', comp.component.name)
                }
            }
            var res = {}
            form.fields.forEach(element=>{
                if(element.type.startsWith('cw_')){
                    var props = {};
                    element.cwSettings.properties.condition.forEach(property=>{
                        if(!element.cwSettings.fields.statements.includes(property.name) && element.cwSettings.fields.statements.length!==0){
                            return;
                        }
                        props[property.name] = `{${element.label}:${element.id}.${property.name}}`;
                    });
                    res[element.label] = props;
                }else if(element.type === 'name'){
                    res[element.label] = {
                        'Prefix': `{${element.label}.prefix:${element.id}.2}`,
                        'First Name': `{${element.label}.first:${element.id}.3}`,
                        'Middle Name': `{${element.label}.middle:${element.id}.4}`,
                        'Last Name': `{${element.label}.last:${element.id}.6}`,
                        'Suffix': `{${element.label}.suffix:${element.id}.8}`,
                    };
                }else if(element.type === 'address'){
                    res[element.label] = {
                        'Street Address': `{${element.label}.address1:${element.id}.1}`,
                        'Address Line 2': `{${element.label}.address2:${element.id}.2}`,
                        'City': `{${element.label}.city:${element.id}.3}`,
                        'State': `{${element.label}.state:${element.id}.4}`,
                        'Zip': `{${element.label}.zip:${element.id}.5}`,
                        'Country': `{${element.label}.country:${element.id}.6}`,
                    };
                } else if (element.type === 'checkbox') {
                    res[element.label] = element.inputs.reduce((obj, item) => ({...obj, [item.label]: `{${element.label}:${item.id}}`}), {})
                }
                else{
                    res[element.label] = `{${element.label}:${element.id}}`;
                }
            })
            commit('changeMT',res);
        },
        async addAction(context, data){
            await context.dispatch('fetchProperties',data.component.name);
            const request = await fetch(`${apiPath}/common/get-required.php?component=${data.component.name}`);
            data['required']  = await request.json();
            context.commit('addAction',data);
            context.dispatch('updateMT');
        },

        async changeSelectedAction(context, data){
            const request = await fetch(`${apiPath}/common/get-required.php?component=${data.component.name}`);
            data['required']  = await request.json();
            context.commit('changeSelectedAction',{index: data['index'],required: data['required']});
        }

    }
})

export default store;
