/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/

Vue.component('merge-tag-u', {
    data: function () {
        return {
            display: false,
            isExpanded: {}
        };
    },
    computed: {
        ...Vuex.mapState(['MergeTags']),
    },
    methods: {
        expanded: function (item) {
            if (typeof this.MergeTags[item] != "object") {
                return false;
            }
            if (this.isExpanded[item] === undefined) {
                this.isExpanded[item] = false;
                return false;
            }
            return this.isExpanded[item];
        },
        selectTag: function (item) {
            if (typeof this.MergeTags[item] == "object") {
                if (this.isExpanded[item] === undefined) {
                    this.isExpanded[item] = true;
                }
                this.isExpanded[item] = !this.isExpanded[item];
            } else {
                this.$emit('chosen', this.MergeTags[item]);
                this.display = false;
            }

            this.$forceUpdate();
        },
        selectSubTag: function (tag) {
            this.$emit('chosen', tag);
            this.display = false;
        },
        toggleMergeTag: function() {
            this.display = !this.display;
            this.isExpanded = {};
        },
        closeMergeTag: function() {
            this.display = false;
        },
        isObject: function(item) {
            return typeof this.MergeTags[item] == 'object';
        },
        getExpandedClass: function(item) {
            if (this.isExpanded[item]) {
                return 'fa-chevron-up';
            } else {
                return 'fa-chevron-down';
            }
        }
    },
    template: `
        <div class="cwf_mt_anchor">
            <a class="cwf_mt_button" @click="toggleMergeTag"></a>
            <div v-if="display" class="cwf_mt_relative">
                <div class="cwf_mt_list">
                    <ul v-for="item in Object.keys(MergeTags)" :key="item">
                        <li>
                            <a @click="selectTag(item)">{{item}}
                                <span v-if="isObject(item)" class="cwf_mt_object">
                                    <i class="fa" :class="getExpandedClass(item)"></i>
                                </span>
                            </a>
                        </li>
                        <li v-if="expanded(item)" v-for="tag in Object.keys(MergeTags[item])" :key="item + tag">
                            <a class="cwf_subtag" @click="selectSubTag(MergeTags[item][tag])">{{tag}}</a>
                        </li>
                    </ul>
                </div>
            </div>
        </div>
    `
});