/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/

const apiPath = '/../wp-content/plugins/gravitywise/includes/api';
const defaultJSON = {
    "cwComponentField": {
        "name": "",
        "endpoint": ""
    },
    "showAllComponents": false,
    "requiredProperties":[
    ],
    "conditions": {
        "conjunction": "ANY",
        "active": false,
        "statements": [{
            "property": "",
            "operator": "=",
            "value": ""
        }]
    },
    "childConditions": {
        "conjunction": "ANY",
        "active": false,
        "statements": [{
            "property": "",
            "operator": "=",
            "value": ""
        }]
    },
    "customFieldConditions": {
        "conjunction": "ANY",
        "active": false,
        "statements": [{
            "property": "",
            "operator": "=",
            "value": ""
        }]
    },
    "orderBy": {
        "active": false,
        "statements": [{
            "property": "Id",
            "order": "ASC"
        }]
    },
    "tableSettings": {
        "active": false,
        "statements": []
    },
    "fields": {
        "active": false,
        "statements": []
    },
    "resultsViewTemplate": {
        "active": false,
        "template": ""
    },
    "allowMultiple": {
        "active": false
    },
    "limitResults": 25
};


const store = new Vuex.Store({
    state: {
        Components: [],
        EnabledSections: {
            basicSettings: false,
            tableSettings: false,
            noSettings: true
        },
        Settings: {
            cwComponentField: {
                name: "",
                endpoint: ""
            },
            showAllComponents: false,
            requiredProperties:[
            ],
            conditions: {
                conjunction: "ANY",
                active: false,
                statements: [{
                    property: "",
                    operator: "=",
                    value: ""
                }]
            },
            childConditions: {
                conjunction: "ANY",
                active: false,
                statements: [{
                    property: "",
                    operator: "=",
                    value: ""
                }]
            },
            customFieldConditions: {
                conjunction: "ANY",
                active: false,
                statements: [{
                    property: "",
                    operator: "=",
                    value: ""
                }]
            },
            orderBy: {
                active: false,
                statements: [{
                    property: "Id",
                    order: "ASC"
                }]
            },
            tableSettings: {
                active: false,
                statements: []
            },
            fields: {
                active: false,
                statements: []
            },
            resultsViewTemplate: {
                active: false,
                template: ""
            },
            allowMultiple: {
                active: false
            },
            limitResults: 25
        },
        Fields: [],
        MergeTags: {},
        Properties: {condition: {}, child: {}, custom: {}},
        APIConfigured: false
    },
    mutations: {
        changeComponents(state,components){
            state.Components = components;
        },
        changeFields(state, fields){
            state.Fields = fields;
        },
        fieldChanged(state,field){
                const isCWField = field.type.startsWith('cw_');
                state.EnabledSections['basicSettings']= isCWField;
                state.EnabledSections['tableSettings']= field.type === 'cw_table';
                state.EnabledSections['noSettings']=!isCWField;
                if(isCWField){
                    state.Settings = field.cwSettings;
                    if(typeof state.Settings !=="object"){
                        state.Settings = defaultJSON;
                        field.cwSettings = defaultJSON;
                    }
                }else{
                    state.Settings = null;
                }
        },
        changeSelectedComponent(state,value){
            state.Settings.cwComponentField = value;
            state.Settings.requiredProperties = [];

            const regex = /{(\w+)}/g;
            var matches = [];
            while(matches = regex.exec(state.Settings.cwComponentField.endpoint)){
                state.Settings.requiredProperties.push({property: matches[1], value: ''});
            }
        },
        changeMT(state, value) {
            state.MergeTags = value;
        },
        changeProperties(state,value){
            state.Properties = value;
            state.Settings.properties = value;
        },
        changeAPIConfigured(state, value){
            state.APIConfigured = value;
        }
    },
    getters: {

    },
    actions: {
        async fetchComponents(context,all){
            const request = await fetch(`${apiPath}/common/get-components.php?all=${all}`);
            context.commit('changeComponents',await request.json());
        },
        async fetchFields(context,component){
            if (component) {
                const request = await  fetch(`${apiPath}/common/get-fields.php?component=${component}`);
                const reqJson = await request.json();
                context.commit('changeFields',Object.keys(reqJson));
            }
        },
        async changeComponent(context,value){
            context.commit('changeSelectedComponent',value);
            context.dispatch('fetchFields',value.name);
            context.dispatch('fetchProperties',value.name);
        },
        async updateMT(context){
            var res = {}
            form.fields.forEach(element=>{
                if(element.type.startsWith('cw_')){
                    var props = {};
                    element.cwSettings.properties.condition.forEach(property=>{
                        props[property.name] = `{${element.label}:${element.id}.${property.name}}`;
                    });
                    res[element.label] = props
                }else{
                    res[element.label] = `{${element.label}:${element.id}}`;
                }
            })
            context.commit('changeMT',res);
        },
        async fetchProperties(context,component){
            const request = await  fetch(`${apiPath}/common/get-properties.php?component=${component}`);
            const properties = await request.json();
            const result = {'condition':[],'child':[],'custom':[]};
            Object.keys(properties).forEach(category=>{
                Object.keys(properties[category]).forEach(element=>{
                        // console.log(category);
                        result[category].push({name: element, type: properties[category][element]});

                })
            })
            context.commit('changeProperties',result);
        },
        async verifyAPI(context) {
            const request = await fetch(`${apiPath}/common/verify-api.php`);
            const apiVerified = await request.json();
            context.commit('changeAPIConfigured', apiVerified);
        },
    }
})

export default store;
