/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/

import '/../wp-content/plugins/gravitywise/public/js/vue/retry/components/atoms/json-viewer.js';

var dissolveObjectPath = function (object){
    var res = {};
    Object.keys(object).forEach(el=>{
        if(typeof object[el] ==="object"){
            var obj = dissolveObjectPath(object[el]);
            Object.keys(obj).forEach(child=>{
                res[`${el}.${child}`] = obj[child];
            })
        }else{
            res[el] = object[el];
        }
    });
    return res;
}

var modifyObjectByPath = function (object, path, value){
    var pathParts = path.split('.');
    if(pathParts.length > 1){
        object[pathParts[0]] = modifyObjectByPath(object[pathParts[0]],pathParts.slice(1).join('.'),value);
    }else{
        object[pathParts[0]] = value;
    }
    return object;
}


Vue.component('action-card', {
    props: ['actionid'],
    data: function () {
        return {
            newValue: null,
            recalculateDirty: 0,
        }
    },
    computed: {
        ...Vuex.mapState(['selectedEntry']),
        action: function(){
            return this.selectedEntry.actions[this.actionid];
        },
        isDirty: function(){
            return JSON.stringify(dissolveObjectPath(JSON.parse(this.action.value))) != JSON.stringify(this.newValueDissolved);
        },
        meta: function () {
            return JSON.parse(this.action.meta);
        },
        requestFields: function (){
            return dissolveObjectPath(this.meta.request);
        },
        newValueDissolved: function (){
            return dissolveObjectPath(this.newValue);
        }


    },
    methods: {
        setValueByPath: function (path,value){
            this.newValue = modifyObjectByPath(this.newValue,path,value);
            this.$forceUpdate();
        },
        saveData: async function(){

            var copy = Object.assign({},this.action);
            copy.value = this.newValue;
            await this.$store.dispatch('updateAction',copy);
            if (this.$refs.save) {
                this.$refs.save.finishLoad();
            }
        },
        retry: async function(force){
            var copy = Object.assign({force: force},this.action);
            copy.value = this.newValue;
            await this.$store.dispatch('retryAction',copy);
            this.resetValue();
            this.$store.dispatch('fetchEntries');
            if (this.$refs.retry) {
                this.$refs.retry.finishLoad();
            }
            if (this.$refs.rebuildRetry) {
                this.$refs.rebuildRetry.finishLoad();
            }
        },
        merge: async function(merge){
            var copy = Object.assign({merge: merge},this.action);
            copy.value = this.newValue;
            await this.$store.dispatch('mergeAction',copy);
            this.resetValue();
            this.$store.dispatch('fetchEntries');
            if (this.$refs.amerge) {
                this.$refs.amerge.finishLoad();

                if (this.$refs.rmerge) {
                    this.$refs.rmerge.finishLoad();
                }
            }else{
                if (this.$refs.retry) {
                    this.$refs.retry.finishLoad();
                }
                if (this.$refs.rebuildRetry) {
                    this.$refs.rebuildRetry.finishLoad();
                }
            }
        },
        SaveRetry: async function(){
            await this.saveData();
            await this.retry(false);
            if (this.$refs.saveRetry) {
                this.$refs.saveRetry.finishLoad();
            }
        },
        resetValue: function(){
            this.newValue = JSON.parse(this.action.value);
            this.$emit('update',this.action.status);
        },
        dismiss: async function(){
            var copy = Object.assign({},this.action);
            copy.status = 'Dismissed';
            await this.$store.dispatch('updateAction',copy);
            if (this.$refs.dismiss) {
                this.$refs.dismiss.finishLoad();
            }
        },

        getPermissions: function(permission, callback){
            navigator.permissions.query({name: "clipboard-write"}).then(result => {
                if (result.state == "granted" || result.state == "prompt") {
                    callback(true);          
                }
            }, reject=>{
                callback(false);
            });
        
        },  
        copyJSON: function(json){                
            this.getPermissions('clipboard-write',(perm)=>{
                
                if(perm){
                    navigator.clipboard.writeText(json);
                }else{
                    this.$refs.copyModal.displayModal(json);
                }
            });
        }
    },
    created: function(){
        this.resetValue();
    },
    template: `
    <div >
        <h3>Component: {{meta.endpoint}}</h3>
        <table class="fr_details_table">
            <tr>
                <th colspan="3">
                    Action Settings
                </th>
            </tr>
            <tr>
                <th>
                    Prop Name
                </th>
                <th>
                    Feed Settings
                </th>
                <th>
                    Submitted Value
                </th>
            </tr>
            <tr v-for="(field, name,i) in requestFields">
                <td>
                    {{name}}
                </td>
                <td>
                    {{field}}
                </td>
                <td>
                    <input type="text" :value="newValueDissolved[name]" @input="setValueByPath(name,$event.target.value)" @change="recalculateDirty++">
                </td>
            </tr>

        </table>
        <div>
            <copy-modal ref="copyModal"></copy-modal>
            <div v-if="action.error_code!=null && action.error_code!='{}'">
                <h3>Response from ConnectWise:</h3>
                    <button class="button button-secondary" @click="$emit('expand')" style="margin-left:1.5rem" type="button">Expand All</button>
                    <button class="button button-secondary" @click="$emit('collapse')" type="button">Collapse All</button>
                    <ibutton @click.native="copyJSON(action.error_code)" icon="copy">Copy Response</ibutton>
                    <json-viewer :json="{Response: JSON.parse(action.error_code||'{}')}"></json-viewer>
                <br>
            </div>
            <div v-if="action.merge_id!=0 && action.has_merged==0">
                
                <h3>Duplicate Details</h3>
                <button class="button button-secondary" @click="$emit('expand')" style="margin-left:1.5rem" type="button">Expand All</button>
                <button class="button button-secondary" @click="$emit('collapse')" type="button">Collapse All</button>
                <json-viewer :json="{Duplicate: JSON.parse(action.merge_json)}"></json-viewer>
            </div>
            <div style="display:flex; flex-direction:row-reverse;" v-if="action.has_merged==0 && action.merge_id!=0">
                <async-button class="button button-primary"  @click="merge(false)" ref="rmerge">Reject Merge</async-button>
                <async-button class="button button-primary"  @click="merge(true)" ref="amerge">Approve Merge</async-button>
            </div>
            <div style="display:flex; flex-direction:row-reverse;" v-if="action.merge_id==0 || action.has_merged==1">
                <async-button class="button button-primary"  @click="dismiss()" ref="dismiss">Dismiss</async-button>
                <async-button class="button button-primary"  @click="retry(true)" ref="rebuildRetry">Rebuild and Retry</async-button>
                <async-button class="button button-primary"  @click="retry(false)" ref="retry">Retry</async-button>
                <async-button class="button button-primary"  @click="SaveRetry()" :disabled="!isDirty" ref="saveRetry">Save and Retry</async-button>
                <async-button class="button button-primary"  @click="saveData()" :disabled="!isDirty" ref="save">Save</async-button>
            </div>
            <br>
        </div>
    </div>
    `,
});