/************************************************************************************************
 * © Copyright 2021, BizZToolz
 * Use of this code is permitted under the terms of the GNU Lesser General Public License
 * https://www.fsf.org/
 ************************************************************************************************/

import '/../wp-content/plugins/gravitywise/public/js/vue/retry/components/molecules/entry-details.js';
import '/../wp-content/plugins/gravitywise/public/js/vue/retry/components/molecules/entry.js';
import '/../wp-content/plugins/gravitywise/public/js/vue/retry/components/atoms/retry-header.js';
import '/../wp-content/plugins/gravitywise/public/js/vue/retry/components/atoms/filter-dropdown.js';

Vue.component('retry-table', {
    data: function () {
        return {
            selected: -1,
            entriesPer: 10,
            page: 1,
        }
    },
    created: function(){
        this.$store.dispatch('fetchEntries');
    },
    methods: {
        clearSelected: function(){
            if (this.selectedEntry != undefined) {
                this.$store.commit('clearSelected');
            }
        },
        clearSelectedID: function(){
            this.selected = -1;
        },
        toggleSelectedID: function(i) {
            this.selected = (this.selected == i) ? -1 : i;
        }
    },
    computed: {
        ...Vuex.mapState(['entries','selectedEntry']),
        ...Vuex.mapGetters(['filteredEntries']),
        feedsBefore: function () {
            if (this.selected == -1) {
                return this.pageEntries;
            }
            return this.pageEntries.slice(0, this.selected + 1);
        },
        feedsAfter: function () {
            if (this.selected == -1) {
                return [];
            }
            return this.pageEntries.slice(this.selected + 1, this.entriesPer);
        },
        selectedID: function () {
            if (this.selected == -1) {
                return this.selected;
            }
            return this.pageEntries[this.selected].entry_id;
        },
        pageEntries: function () {
            var res = [];
            for (let index = (this.page - 1) * this.entriesPer; index < Math.min((this.page) * this.entriesPer, this.filteredEntries.length); index++) {
                const element = this.filteredEntries[index];
                res.push(element);
            }
            return res;
        },
        pages: function () {
            return Math.ceil(this.filteredEntries.length / this.entriesPer);
        },
        getAllFeeds: function(){
            var feeds = {};
            this.entries.forEach(entry => {
                feeds[entry.feed_id] = JSON.parse(entry.feed_meta)['name'];
            });
            return feeds;
        }
    },



    template: `
    <div class="fr_header">
        <table class="fr_entry_matrix">
            <thead is="retry-header" @clear="clearSelectedID" :filters="['Success','Fail','Wait','Duplicate','No Connection']">
            </thead>
            <tbody>
                <tr>
                    <th>
                    </th>
                    <th class="fr_filter_box_anchor">
                        Feed<filter-dropdown :feeds="getAllFeeds"></filter-dropdown>
                    </th>
                    <th>
                        Form
                    </th>
                    <th>
                        Entry
                    </th>
                    <th>
                        Date
                    </th>
                    <th>
                        Steps
                    </th>
                    <th>
                        Status
                    </th>
                    <th>
                        Actions
                    </th>
                </tr>
                <tr v-show="this.filteredEntries.length === 0">
                    <td colspan="8" class="fr_no_results">No results found</td>
                </tr>
                <tr is="entry" v-for="ent, i in feedsBefore" :entry="ent" @selected="clearSelected(); toggleSelectedID(i);" :key="ent.entry_id"></tr>
                
                <tr is="entry-details" v-if="this.selectedID != undefined" :selected="selectedID"></tr>
                
                <tr is="entry" v-for="ent, i in feedsAfter" :entry="ent" @selected="clearSelected(); toggleSelectedID(feedsBefore.length + i);" :key="ent.entry_id"></tr>
            </tbody>
            <tfoot>
                <tr>
                    <th colspan="8">
                        <p class="fr_page_text">
                        Page
                        <a v-for="p in pages" class="fr_pagination" :class="{fr_selected:p==page}" @click="page=p;selected=-1">{{p}}</a>
                        </p>
                    </th>
                </tr>
            </tfoot>
        </table>
    </div>
    `,
});