const apiPath = '/../wp-content/plugins/gravitywise/includes/api';

const store = new Vuex.Store({
    state: {
        actions: [],
        entries: [],
        selectedEntry: undefined,
        filters: {
            status: function(){return true;},
            feed: function(){return true;},
            dismissed: function(){return true;}
        }
    },
    mutations: {
        changeEntries(state, entries) {
            var temp = [];
            for (let i = 0; i < entries.length; i++) {
                const element = entries[i];
                if(element.status != 'Scrubbed'){
                    temp.push(element);
                }
            }
            state.entries = temp;
        },
        changeActions(state, actions) {
            // console.log(actions);
            for (let i = 0; i < state.selectedEntry.actions.length; i++) {
                // console.log(state.selectedEntry.actions[i].id + " " + actions[0].id);
                if(state.selectedEntry.actions[i].id == actions[0].id){
                    Vue.set(state.selectedEntry.actions,i,actions[0]);
                }
                
            }
        },
        changeSelectedEntry(state,entry){
            state.selectedEntry = entry
        },
        clearSelected(state){
            state.selectedEntry = undefined;
        },
        dismissEntry(state,entry){
            for (let i = 0; i < state.entries.length; i++) {
                const element = state.entries[i];
                if(element.id == entry){
                    state.entries[i].dismissed=1;
                }
            }
        },
        restoreEntry(state,entry){
            for (let i = 0; i < state.entries.length; i++) {
                const element = state.entries[i];
                if(element.id == entry){
                    state.entries[i].dismissed=0;
                }
            }
        },
        deleteEntry(state,entry){
            for (let i = 0; i < state.entries.length; i++) {
                const element = state.entries[i];
                if(element.id == entry){
                    state.entries.splice(i,1);
                }
            }
        },
        changeFilters(state,{filter, value}){
            state.filters[filter] = value;
        }
    },
    getters: {
        filteredEntries(state) {
            var temp = [];
            state.entries.forEach(element => {
                var passed = true;
                Object.keys(state.filters).forEach(key => {
                    passed = passed && state.filters[key](element);
                });
                if(passed){
                    temp.push(element);
                }
            });
            return temp;
        }
    },
    actions: {
        async fetchEntries(context) {
            var response = await fetch(`${apiPath}/feed-retry/get-entries.php`)
            context.commit('changeEntries', await response.json());
            
        },
        async dismissEntry(context, entry) {
            const fd = new FormData();
            fd.append('id',entry.id);
            fd.append('op',entry.op)
            var response = await fetch(`${apiPath}/feed-retry/delete-action.php`,{
                method: 'POST',
                body: fd
            });
            if(await response.text() == 'Success'){
                if(entry.op == 'delete'){
                    context.commit('deleteEntry',entry.id);
                }else if(entry.op == 'dismiss'){
                    context.commit('dismissEntry',entry.id);
                }else{
                    context.commit('restoreEntry',entry.id);
                }
            }

        },
        fetchDetails(context, {entryId,feedId}) {
            fetch(`${apiPath}/feed-retry/get-details.php?eid=${entryId}&fid=${feedId}`).then(async (response) => {
                context.commit('changeSelectedEntry', await response.json());
            });

        },
        async updateAction(context, action) {
            var response = await fetch(`${apiPath}/feed-retry/update-action.php`, {
                method: 'POST',
                body: JSON.stringify(action)
            })
            context.commit('changeActions', await response.json());
        
        },
        async retryAction(context, { id,entry_attempt_id, force }) {
            const fd = new FormData();
            fd.append('id', id);
            fd.append('entry_id', context.state.selectedEntry.details.id);
            fd.append('force',force);
            var req = await fetch(`${apiPath}/feed-retry/retry-action.php`,{
                method: 'POST',
                body: fd
            })
            await context.commit('changeActions', await req.json());
        },
        async mergeAction(context, { id,entry_attempt_id, merge }) {
            const fd = new FormData();
            fd.append('id', id);
            fd.append('entry_id', context.state.selectedEntry.details.id);
            fd.append('merge',merge);
            var req = await fetch(`${apiPath}/feed-retry/merge-action.php`,{
                method: 'POST',
                body: fd
            })
            await context.commit('changeActions', await req.json());
        },

    }
})

export default store;
